#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor
# chapter 13, example 6.
#
# Boolean engine.  Derived from example 13.5.
# The smaller duck stays still while the bigger duck moves,
# and starts moving as soon as the bigger duck stops.
#

from __future__ import print_function
import sys

from pivy.coin import *
from pivy.sogui import *

# This routine is called for every mouse button event.
def myMousePressCB(gate, eventCB):
    event = eventCB.getEvent()

    # Check for mouse button being pressed
    if SoMouseButtonEvent.isButtonPressEvent(event, SoMouseButtonEvent.ANY):

        # Toggle the gate that controls the duck motion
        if gate.enable.getValue():
            gate.enable = FALSE
        else:
            gate.enable = TRUE

        eventCB.setHandled()


def main():
    # Print out usage message
    print("Only one duck can move at a time.")
    print("Click the left mouse button to toggle between the two ducks.")

    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])  
    if myWindow == None: sys.exit(1)     

    root = SoSeparator()

    # Add a camera and light
    myCamera = SoPerspectiveCamera()
    myCamera.position = (0., -4., 8.0)
    myCamera.heightAngle = M_PI/2.5
    myCamera.nearDistance = 1.0
    myCamera.farDistance = 15.0
    root.addChild(myCamera)
    root.addChild(SoDirectionalLight())

    # Rotate scene slightly to get better view
    globalRotXYZ = SoRotationXYZ()
    globalRotXYZ.axis = SoRotationXYZ.X
    globalRotXYZ.angle = M_PI/9
    root.addChild(globalRotXYZ)

    # Pond group
    pond = SoSeparator()
    root.addChild(pond)
    pondTranslation = SoTranslation()
    pondTranslation.translation = (0., -6.725, 0.)
    pond.addChild(pondTranslation)
    # water
    waterMaterial = SoMaterial()
    waterMaterial.diffuseColor = (0., 0.3, 0.8)
    pond.addChild(waterMaterial)
    waterCylinder = SoCylinder()
    waterCylinder.radius = 4.0
    waterCylinder.height = 0.5
    pond.addChild(waterCylinder)
    # rock
    rockMaterial = SoMaterial()
    rockMaterial.diffuseColor = (0.8, 0.23, 0.03)
    pond.addChild(rockMaterial)
    rockSphere = SoSphere()
    rockSphere.radius = 0.9
    pond.addChild(rockSphere)

    # Read the duck object from a file and add to the group
    myInput = SoInput()
    if not myInput.openFile("duck.iv"):
        sys.exit(1)
    duckObject = SoDB.readAll(myInput)
    if duckObject == None:
        sys.exit(1)

#############################################################
# CODE FOR The Inventor Mentor STARTS HERE  

    # Bigger duck group
    bigDuck = SoSeparator()
    root.addChild(bigDuck)
    bigDuckRotXYZ = SoRotationXYZ()
    bigDuck.addChild(bigDuckRotXYZ)
    bigInitialTransform = SoTransform()
    bigInitialTransform.translation = (0., 0., 3.5)
    bigInitialTransform.scaleFactor = (6., 6., 6.)
    bigDuck.addChild(bigInitialTransform)
    bigDuck.addChild(duckObject)

    # Smaller duck group
    smallDuck = SoSeparator()
    root.addChild(smallDuck)
    smallDuckRotXYZ = SoRotationXYZ()
    smallDuck.addChild(smallDuckRotXYZ)
    smallInitialTransform = SoTransform()
    smallInitialTransform.translation = (0., -2.24, 1.5)
    smallInitialTransform.scaleFactor = (4., 4., 4.)
    smallDuck.addChild(smallInitialTransform)
    smallDuck.addChild(duckObject)

    # Use a gate engine to start/stop the rotation of 
    # the bigger duck.
    bigDuckGate = SoGate(SoMFFloat.getClassTypeId())
    bigDuckTime = SoElapsedTime()
    bigDuckGate.input.connectFrom(bigDuckTime.timeOut) 
    bigDuckRotXYZ.axis = SoRotationXYZ.Y  # Y axis
    bigDuckRotXYZ.angle.connectFrom(bigDuckGate.output)

    # Each mouse button press will enable/disable the gate 
    # controlling the bigger duck.
    myEventCB = SoEventCallback()
    myEventCB.addEventCallback(SoMouseButtonEvent.getClassTypeId(),
                               myMousePressCB, bigDuckGate)
    root.addChild(myEventCB)

    # Use a Boolean engine to make the rotation of the smaller
    # duck depend on the bigger duck.  The smaller duck moves
    # only when the bigger duck is still.
    myBoolean = SoBoolOperation()
    myBoolean.a.connectFrom(bigDuckGate.enable)
    myBoolean.operation = SoBoolOperation.NOT_A

    smallDuckGate = SoGate(SoMFFloat.getClassTypeId())
    smallDuckTime = SoElapsedTime()
    smallDuckGate.input.connectFrom(smallDuckTime.timeOut) 
    smallDuckGate.enable.connectFrom(myBoolean.output) 
    smallDuckRotXYZ.axis = SoRotationXYZ.Y  # Y axis
    smallDuckRotXYZ.angle.connectFrom(smallDuckGate.output)

# CODE FOR The Inventor Mentor ENDS HERE
#############################################################

    myRenderArea = SoGuiRenderArea(myWindow)
    myRenderArea.setSceneGraph(root)
    myRenderArea.setTitle("Duck and Duckling")
    myRenderArea.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2008 Kongsberg SIM
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

import sys

from PySide2.QtWidgets import QApplication

from pivy.coin import SoSeparator
from pivy.coin import SoBaseColor
from pivy.coin import SbColor
from pivy.coin import SoCone

from pivy.quarter import QuarterWidget

def main():
    app = QApplication(sys.argv)

    root = SoSeparator()
    col = SoBaseColor()
    col.rgb = SbColor(1, 1, 0)
    root.addChild(col)
    root.addChild(SoCone())

    viewer = QuarterWidget()
    viewer.setSceneGraph(root)

    viewer.setWindowTitle("minimal")
    viewer.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from The Inventor Mentor,
# chapter 9, example 4.
#
# Example of setting up pick actions and using the pick path.
# A couple of objects are displayed.  The program catches 
# mouse button events and determines the mouse position. 
# A pick action is applied and if an object is picked the
# pick path is printed to stdout.
#

import sys

from pivy.coin import *
from pivy.sogui import *

###############################################################
# CODE FOR The Inventor Mentor STARTS HERE

def writePickedPath(root, viewport, cursorPosition):
    myPickAction = SoRayPickAction(viewport)

    # Set an 8-pixel wide region around the pixel
    myPickAction.setPoint(cursorPosition)
    myPickAction.setRadius(8.0)

    # Start a pick traversal
    myPickAction.apply(root)
    myPickedPoint = myPickAction.getPickedPoint()
    if myPickedPoint == None: return FALSE

    # Write out the path to the picked object
    myWriteAction = SoWriteAction()
    myWriteAction.apply(myPickedPoint.getPath())

    return TRUE

# CODE FOR The Inventor Mentor ENDS HERE
###############################################################

# This routine is called for every mouse button event.
def myMousePressCB(userData, eventCB):
    root = userData
    event = eventCB.getEvent()

    # Check for mouse button being pressed  
    if SoMouseButtonEvent.isButtonPressEvent(event, SoMouseButtonEvent.ANY):
        myRegion = eventCB.getAction().getViewportRegion()
        writePickedPath(root, myRegion, event.getPosition(myRegion))
        eventCB.setHandled()

def main():
    myMouseEvent = SoMouseButtonEvent()

    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None:
        sys.exit(1)
    
    root = SoSeparator()

    # Add an event callback to catch mouse button presses.
    # The callback is set up later on.
    myEventCB = SoEventCallback()
    root.addChild(myEventCB)

    # Read object data from a file
    mySceneInput = SoInput()
    if not mySceneInput.openFile("star.iv"):
        sys.exit(1)
    starObject = SoDB.readAll(mySceneInput)
    if starObject == None: sys.exit(1)
    mySceneInput.closeFile()

    # Add two copies of the star object, one white and one red
    myRotation = SoRotationXYZ()
    myRotation.axis = SoRotationXYZ.X
    myRotation.angle = M_PI/2.2  # almost 90 degrees
    root.addChild(myRotation)

    root.addChild(starObject)  # first star object

    myMaterial = SoMaterial()
    myMaterial.diffuseColor = (1.0, 0.0, 0.0)   # red
    root.addChild(myMaterial)
    myTranslation = SoTranslation()
    myTranslation.translation = (1.0, 0.0, 1.0)
    root.addChild(myTranslation)
    root.addChild(starObject)  # second star object

    # Create a render area in which to see our scene graph.
    myViewer = SoGuiExaminerViewer(myWindow)

    # Turn off viewing to allow picking
    myViewer.setViewing(0)

    myViewer.setSceneGraph(root)
    myViewer.setTitle("Pick Actions & Paths")
    myViewer.show()

    # Set up the event callback. We want to pass the root of the
    # entire scene graph (including the camera) as the userData,
    # so we get the scene manager's version of the scene graph
    # root.
    myEventCB.addEventCallback(SoMouseButtonEvent.getClassTypeId(),
                               myMousePressCB,
                               myViewer.getSceneManager().getSceneGraph())

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 14, example 3.
#
# This example illustrates the creation of motion hierarchies
# using nodekits by creating a model of a balance-style scale.
#
# It adds an SoEventCallback to the "callback" list in the 
#     nodekit called 'support.'
# The callback will have the following response to events:
# Pressing right arrow key == lower the right pan
# Pressing left arrow key  == lower the left pan
# The pans are lowered by animating three rotations in the 
#     motion hierarchy.
# Use an SoText2Kit to print instructions to the user as part
#     of the scene.
#

import sys

from pivy.coin import *
from pivy.sogui import *

# Callback Function for Animating the Balance Scale.
# -- used to make the balance tip back and forth
# -- Note: this routine is only called in response to KeyPress
#    events since the call 'setEventInterest(KeyPressMask)' is
#    made on the SoEventCallback node that uses it.
# -- The routine checks if the key pressed was left arrow (which
#    is XK_Left in X-windows talk), or right arrow (which is
#    XK_Right)
# -- The balance is made to tip by rotating the beam part of the
#    scale (to tip it) and then compensating (making the strings
#    vertical again) by rotating the string parts in the opposite
#    direction.
def tipTheBalance(support, # The nodekit representing 'support', the
                  # fulcrum of the balance. Passed in during
                  # main routine, below. 
                  eventCB):

    ev = eventCB.getEvent()

    # Which Key was pressed?
    # If Right or Left Arrow key, then continue...
    if SoKeyboardEvent.isKeyPressEvent(ev, SoKeyboardEvent.RIGHT_ARROW) or \
       SoKeyboardEvent.isKeyPressEvent(ev, SoKeyboardEvent.LEFT_ARROW):
        startRot, beamIncrement, stringIncrement = SbRotation(), SbRotation(), SbRotation()
        
        # These three parts are extracted based on knowledge of the
        # motion hierarchy (see the diagram in the main routine.
        beam1   = support.getPart("childList[0]",TRUE)
        string1 = beam1.getPart("childList[0]",TRUE)
        string2 = beam1.getPart("childList[1]",TRUE)

        # Set angular increments to be .1 Radians about the Z-Axis
        # The strings rotate opposite the beam, and the two types
        # of key press produce opposite effects.
        if SoKeyboardEvent.isKeyPressEvent(ev, SoKeyboardEvent.RIGHT_ARROW):
            beamIncrement.setValue(SbVec3f(0, 0, 1), -.1)
            stringIncrement.setValue(SbVec3f(0, 0, 1), .1)
        else:
            beamIncrement.setValue(SbVec3f(0, 0, 1), .1)
            stringIncrement.setValue(SbVec3f(0, 0, 1), -.1)

        # Use SO_GET_PART to find the transform for each of the 
        # rotating parts and modify their rotations.

        xf = beam1.getPart("transform", TRUE)
        startRot = xf.rotation.getValue()
        startRot *= beamIncrement
        xf.rotation = startRot

        xf = string1.getPart("transform", TRUE)
        startRot = xf.rotation.getValue()
        startRot *= stringIncrement
        xf.rotation = startRot

        xf = string2.getPart("transform", TRUE)
        startRot = xf.rotation.getValue()
        startRot *= stringIncrement     
        xf.rotation = startRot

        eventCB.setHandled()

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])  
    if myWindow == None: sys.exit(1)     

    myScene = SoSceneKit()

    myScene.setPart("lightList[0]", SoLightKit())
    myScene.setPart("cameraList[0]", SoCameraKit())
    myScene.setCameraNumber(0)

    # Create the Balance Scale -- put each part in the 
    # childList of its parent, to build up this hierarchy:
    #
    #                    myScene
    #                       |
    #                     support
    #                       |
    #                     beam
    #                       |
    #                   --------
    #                   |       |
    #                string1  string2
    #                   |       |
    #                tray1     tray2

    support = SoShapeKit()
    support.setPart("shape", SoCone())
    support.set("shape { height 3 bottomRadius .3 }")
    myScene.setPart("childList[0]", support)

    beam = SoShapeKit()
    beam.setPart("shape", SoCube())
    beam.set("shape { width 3 height .2 depth .2 }")
    beam.set("transform { translation 0 1.5 0 } ")
    support.setPart("childList[0]", beam)

    string1 = SoShapeKit()
    string1.setPart("shape", SoCylinder())
    string1.set("shape { radius .05 height 2}")
    string1.set("transform { translation -1.5 -1 0 }")
    string1.set("transform { center 0 1 0 }")
    beam.setPart("childList[0]", string1)

    string2 = SoShapeKit()
    string2.setPart("shape", SoCylinder())
    string2.set("shape { radius .05 height 2}")
    string2.set("transform { translation 1.5 -1 0 } ")
    string2.set("transform { center 0 1 0 } ")
    beam.setPart("childList[1]", string2)

    tray1 = SoShapeKit()
    tray1.setPart("shape", SoCylinder())
    tray1.set("shape { radius .75 height .1 }")
    tray1.set("transform { translation 0 -1 0 } ")
    string1.setPart("childList[0]", tray1)

    tray2 = SoShapeKit()
    tray2.setPart("shape", SoCylinder())
    tray2.set("shape { radius .75 height .1 }")
    tray2.set("transform { translation 0 -1 0 } ")
    string2.setPart("childList[0]", tray2)

    # Add EventCallback so Balance Responds to Events
    myCallbackNode = SoEventCallback()
    myCallbackNode.addEventCallback(SoKeyboardEvent.getClassTypeId(),
                                    tipTheBalance, support)
    support.setPart("callbackList[0]", myCallbackNode)

    # Add Instructions as Text in the Scene...
    myText = SoShapeKit()
    myText.setPart("shape", SoText2())
    myText.set("shape { string \"Press Left or Right Arrow Key\" }")
    myText.set("shape { justification CENTER }")
    myText.set("font { name \"Helvetica-Bold\" }")
    myText.set("font { size 16.0 }")
    myText.set("transform { translation 0 -2 0 }")
    myScene.setPart("childList[1]", myText)

    myRenderArea = SoGuiRenderArea(myWindow)

    # Get camera from scene and tell it to viewAll...
    myCamera = myScene.getPart("cameraList[0].camera", TRUE)
    myCamera.viewAll(myScene, myRenderArea.getViewportRegion())

    myRenderArea.setSceneGraph(myScene)
    myRenderArea.setTitle("Balance Scale Made of Nodekits")
    myRenderArea.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# This file is part of a set of example programs for the Coin library.
# Copyright (C) 2000-2003 by Systems in Motion. All rights reserved.
#
#                  <URL:http://www.coin3d.org>
#
# This sourcecode can be redistributed and/or modified under the
# terms of the GNU General Public License version 2 as published by
# the Free Software Foundation. See the file COPYING at the root
# directory of the distribution for more details.
#
# As a special exception, all sourcecode of the demo examples can be
# used for any purpose for licensees of the Coin Professional
# Edition License, without the restrictions of the GNU GPL. See our
# web pages for information about how to acquire a Professional Edition
# License.
#
# Systems in Motion, <URL:http://www.sim.no>, <mailto:support@sim.no>
#

SOGUI_BINDING = "SoQt"

import sys

from pivy.coin import *
from pivy.sogui import *
from pivy.simvoleon import *

import utility

def main():
    window = SoGui.init(sys.argv[0])
    SoVolumeRendering.init()

    root = SoSeparator()

    dim = SbVec3s(64, 64, 64)
    voxeldata = utility.generate8bitVoxelSet(dim)

    # Add SoVolumeData to scene graph
    volumedata = SoVolumeData()
    volumedata.setVolumeData(dim, voxeldata.tostring())
    root.addChild(volumedata)

    # Add TransferFunction (color map) to scene graph
    transfunc = SoTransferFunction()
    root.addChild(transfunc)

    # Add VolumeRender to scene graph
    volrend = SoVolumeRender()
    root.addChild(volrend)

    viewer = SoGuiExaminerViewer(window)
    viewer.setBackgroundColor(SbColor(0.1, 0.3, 0.5))
    viewer.setSceneGraph(root)

    viewer.show()
    SoGui.show(window)
    SoGui.mainLoop()

if __name__ == '__main__':
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

import sys
from random import random

from pivy.coin import *
from pivy.sogui import *

import shapescale

def construct_new_marker(v):
    markerroot = SoSeparator()

    t = SoTranslation()
    t.translation = v
    markerroot.addChild(t)
    
    kit = ShapeScale()
    kit.active = TRUE
    kit.projectedSize = 5.0
  
    # create the marker
    markersep = SoSeparator()
  
    mat = SoBaseColor()
    mat.rgb = (random(), random(), random())
    markersep.addChild(mat)
  
    # marker shape should be unit size, with center in (0.0f, 0.0f, 0.0f)
    cube = SoCube()
    cube.width = 1.0
    cube.height = 1.0
    cube.depth = 1.0
    
    markersep.addChild(cube)
    kit.setPart("shape", markersep)
    markerroot.addChild(kit)

    return markerroot


def event_cb(viewer, n):
    mbe = n.getEvent()

    if (mbe.getButton() == SoMouseButtonEvent.BUTTON1 and
        mbe.getState() == SoButtonEvent.DOWN):

        rp = SoRayPickAction(viewer.getViewportRegion())
        rp.setPoint(mbe.getPosition())
        rp.apply(viewer.getSceneManager().getSceneGraph())

        point = rp.getPickedPoint()
        if point == None:
            print("\n** miss! **\n", file=sys.stderr)
            return

        n.setHandled()

        p = rp.getCurPath()

        for i in range(p.getLength()):
            n = p.getNodeFromTail(i)
            if n.isOfType(SoGroup.getClassTypeId()):
                n.addChild(construct_new_marker(point.getPoint()))
                break
        

def show_instructions():
    print("""
This example program demonstrates the use of the ShapeScale nodekit.
Quick instructions:


  * place the marker by clicking on a shape with the left mouse button
  * hit ESC to toggle back and forth to view mode
  * zoom back and forth to see how the markers stay the same size

""")

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("\nSpecify an Inventor file as argument.\n", file=sys.stderr)
        sys.exit(-1)

    window = SoGui.init(sys.argv[0])
    ShapeScale.initClass() # init our extension nodekit

    ex1 = SoGuiExaminerViewer(window)

    input = SoInput()
    if not input.openFile(sys.argv[1]):
        print("Unable to open file: %s\n" % sys.argv[1], file=sys.stderr)
        sys.exit(-1)

    root = SoDB.readAll(input) 

    if root == None:
        print("Unable to read file: %s\n" % sys.argv[1], file=sys.stderr)
        sys.exit(-1)

    show_instructions()

    newroot = SoSeparator()

    newroot.addChild(root)

    # create event callback and marker nodes
    sep = SoSeparator()
    newroot.addChild(sep)

    ecb = SoEventCallback()
    ecb.addEventCallback(SoMouseButtonEvent.getClassTypeId(), event_cb, ex1)
    sep.addChild(ecb)

    ex1.setSceneGraph(newroot)
    ex1.setTransparencyType(SoGLRenderAction.SORTED_OBJECT_BLEND)
    ex1.setViewing(FALSE)
  
    ex1.show()
    SoGui.show(window)

    SoGui.mainLoop()

    del ex1
    
    sys.exit(0)



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor
# chapter 13, example 3.
#
# Elapsed time engine.
# The output from an elapsed time engine is used to control
# the translation of the object.  The resulting effect is
# that the figure slides across the scene.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])  
    if myWindow == None: sys.exit(1)     

    root = SoSeparator()

    # Add a camera and light
    myCamera = SoPerspectiveCamera()
    myCamera.position = (-2.0, -2.0, 5.0)
    myCamera.heightAngle = M_PI/2.5
    myCamera.nearDistance = 2.0
    myCamera.farDistance = 7.0
    root.addChild(myCamera)
    root.addChild(SoDirectionalLight())

    # Set up transformations
    slideTranslation = SoTranslation()
    root.addChild(slideTranslation)
    initialTransform = SoTransform()
    initialTransform.translation = (-5., 0., 0.)
    initialTransform.scaleFactor = (10., 10., 10.)
    initialTransform.rotation.setValue(SbVec3f(1,0,0), M_PI/2.)
    root.addChild(initialTransform)

    # Read the figure object from a file and add to the scene
    myInput = SoInput()
    if not myInput.openFile("jumpyMan.iv"):
        sys.exit (1)
    figureObject = SoDB.readAll(myInput)
    if figureObject == None:
        sys.exit(1)
    root.addChild(figureObject)

    # Make the X translation value change over time.
    myCounter = SoElapsedTime()
    slideDistance = SoComposeVec3f()
    slideDistance.x.connectFrom(myCounter.timeOut)
    slideTranslation.translation.connectFrom(slideDistance.vector)

    myRenderArea = SoGuiRenderArea(myWindow)
    myRegion = SbViewportRegion(myRenderArea.getSize()) 
    myRenderArea.setSceneGraph(root)
    myRenderArea.setTitle("Sliding Man")
    myRenderArea.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor
# chapter 16, example 5.
#
# This example creates a simple scene graph and attaches a
# browser Examiner viewer to view the data. The camera and
# light in the scene are automatically created by the viewer.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
   
    # Build the viewer in the applications main window
    myViewer = SoGuiExaminerViewer(myWindow)
   
    # Read the geometry from a file and add to the scene
    myInput = SoInput()
    if not myInput.openFile("dogDish.iv"):
        sys.exit(1)
    geomObject = SoDB.readAll(myInput)
    if geomObject == None:
        sys.exit(1)
   
    # Attach the viewer to the scene graph
    myViewer.setSceneGraph(geomObject)
   
    # Show the main window
    myViewer.show()
    SoGui.show(myWindow)

    # Loop forever
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from The Inventor Mentor,
# chapter 9, example 5.
#
# Using a callback for generated primitives.
# A simple scene with a sphere is created.
# A callback is used to write out the triangles that
# form the sphere in the scene.
#

from __future__ import print_function
import sys

from pivy.coin import *

##############################################################
# CODE FOR The Inventor Mentor STARTS HERE

def printVertex(vertex):
    point = vertex.getPoint()
    print("\tCoords     = (%g, %g, %g)" % (point[0], point[1], point[2]))

    normal = vertex.getNormal()
    print("\tNormal     = (%g, %g, %g)" % (normal[0], normal[1], normal[2]))

def printHeaderCallback(void, callbackAction, node):
    print("\n Sphere ")
    # Print the node name (if it exists) and address
    if not not node.getName():
        print('named "%s" ' % node.getName().getString())
    print("at address %r\n" % node.this)

    return SoCallbackAction.CONTINUE

def printTriangleCallback(void, callbackAction, vertex1, vertex2, vertex3):
    print("Triangle:")
    printVertex(vertex1)
    printVertex(vertex2)
    printVertex(vertex3)

def printSpheres(root):
    myAction = SoCallbackAction()
    
    myAction.addPreCallback(SoSphere.getClassTypeId(), printHeaderCallback, None)
    myAction.addTriangleCallback(SoSphere.getClassTypeId(), printTriangleCallback, None)

    myAction.apply(root)
    
# CODE FOR The Inventor Mentor ENDS HERE
##############################################################

def main():
    # Initialize Inventor
    # SoDB.init() invoked automatically upon coin module import

    # Make a scene containing a red sphere
    root = SoSeparator()
    myCamera = SoPerspectiveCamera()
    myMaterial = SoMaterial()
    root.addChild(myCamera)
    root.addChild(SoDirectionalLight())
    myMaterial.diffuseColor = (1.0, 0.0, 0.0)   # Red
    root.addChild(myMaterial)
    root.addChild(SoSphere())

    # Write out the triangles that form the sphere in the scene
    printSpheres(root)

    return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 7, example 3.
#
# This example illustrates using texture functions to
# generate texture coordinates on a sphere.
# It draws three texture mapped spheres, each with a 
# different repeat frequency as defined by the fields of the 
# SoTextureCoordinatePlane node.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoSeparator()

    # Choose a texture.
    faceTexture = SoTexture2()
    root.addChild(faceTexture)
    faceTexture.filename = "sillyFace.rgb"

    # Make the diffuse color pure white
    myMaterial = SoMaterial()
    myMaterial.diffuseColor = (1,1,1)
    root.addChild(myMaterial)

    # This texture2Transform centers the texture about (0,0,0) 
    myTexXf = SoTexture2Transform()
    myTexXf.translation = (.5,.5)
    root.addChild(myTexXf)

    # Define a texture coordinate plane node.  This one will 
    # repeat with a frequency of two times per unit length.
    # Add a sphere for it to affect.
    texPlane1 = SoTextureCoordinatePlane()
    texPlane1.directionS = (2,0,0)
    texPlane1.directionT = (0,2,0)
    root.addChild(texPlane1)
    root.addChild(SoSphere())

    # A translation node for spacing the three spheres.
    myTranslation = SoTranslation()
    myTranslation.translation = (2.5,0,0)

    # Create a second sphere with a repeat frequency of 1.
    texPlane2 = SoTextureCoordinatePlane()
    texPlane2.directionS = (1,0,0)
    texPlane2.directionT = (0,1,0)
    root.addChild(myTranslation)
    root.addChild(texPlane2)
    root.addChild(SoSphere())

    # The third sphere has a repeat frequency of .5
    texPlane3 = SoTextureCoordinatePlane()
    texPlane3.directionS = (.5,0,0)
    texPlane3.directionT = (0,.5,0)
    root.addChild(myTranslation)
    root.addChild(texPlane3)
    root.addChild(SoSphere())

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Texture Coordinate Plane")

    # In Inventor 2.1, if the machine does not have hardware texture
    # mapping, we must override the default drawStyle to display textures.
    # myViewer.setDrawStyle(SoGuiViewer.STILL, SoGuiViewer.VIEW_AS_IS)
    
    myViewer.show()
    myViewer.viewAll()
    
    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 4, example 1.
#
# Camera example.  
# A blinker node is used to switch between three 
# different views of the same scene. The cameras are 
# switched once per second.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoSeparator()

    # Create a blinker node and put it in the scene. A blinker
    # switches between its children at timed intervals.
    myBlinker = SoBlinker()
    root.addChild(myBlinker)

    # Create three cameras. Their positions will be set later.
    # This is because the viewAll method depends on the size
    # of the render area, which has not been created yet.
    orthoViewAll = SoOrthographicCamera()
    perspViewAll = SoPerspectiveCamera()
    perspOffCenter = SoPerspectiveCamera()
    myBlinker.addChild(orthoViewAll)
    myBlinker.addChild(perspViewAll)
    myBlinker.addChild(perspOffCenter)

    # Create a light
    root.addChild(SoDirectionalLight())

    # Read the object from a file and add to the scene
    myInput = SoInput()
    if not myInput.openFile("parkbench.iv"):
        sys.exit(1)

    fileContents = SoDB.readAll(myInput)
    if fileContents == None:
        sys.exit(1)

    myMaterial = SoMaterial()
    myMaterial.diffuseColor = (0.8, 0.23, 0.03) 
    root.addChild(myMaterial)
    root.addChild(fileContents)

    myRenderArea = SoGuiRenderArea(myWindow)

    # Establish camera positions. 
    # First do a viewAll on all three cameras.  
    # Then modify the position of the off-center camera.
    myRegion = SbViewportRegion(myRenderArea.getSize())
    orthoViewAll.viewAll(root, myRegion)
    perspViewAll.viewAll(root, myRegion)
    perspOffCenter.viewAll(root, myRegion)
    initialPos = perspOffCenter.position.getValue()
    x,y,z = initialPos.getValue()
    perspOffCenter.position = (x+x/2., y+y/2., z+z/4.)

    myRenderArea.setSceneGraph(root)
    myRenderArea.setTitle("Cameras")
    myRenderArea.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 16, example 4.
#
# This example builds a render area and Material Editor within 
# a window supplied by the application. It uses a Motif form 
# widget to lay both components inside the same window.  
# It attaches the editor to the material of an object.
#

from __future__ import print_function
import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
   
    # Build the form to hold both components
    myForm = QtCreateWidget("Form", xmFormWidgetClass, myWindow, None, 0)
   
    # Build the render area and Material Editor
    myRenderArea = SoGuiRenderArea(myForm)
    myRenderArea.setSize(SbVec2s(200, 200))
    myEditor = SoGuiMaterialEditor(myForm)
   
    # Layout the components within the form
    args = []
    QtSetArg(args[0], XmNtopAttachment,    XmATTACH_FORM)
    QtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM)
    QtSetArg(args[2], XmNleftAttachment,   XmATTACH_FORM) 
    QtSetArg(args[3], XmNrightAttachment,  XmATTACH_POSITION)
    QtSetArg(args[4], XmNrightPosition,    40)
    QtSetValues(myRenderArea.getWidget(), args, 5)
    QtSetArg(args[2], XmNrightAttachment,  XmATTACH_FORM) 
    QtSetArg(args[3], XmNleftAttachment,   XmATTACH_POSITION)
    QtSetArg(args[4], XmNleftPosition,     41) 
    QtSetValues(myEditor.getWidget(), args, 5)
    
    # Create a scene graph
    root = SoSeparator()
    myCamera = SoPerspectiveCamera()
    myMaterial = SoMaterial()
   
    myCamera.position = (0.212482, -0.881014, 2.5)
    myCamera.heightAngle = M_PI/4
    root.addChild(myCamera)
    root.addChild(SoDirectionalLight())
    root.addChild(myMaterial)

    # Read the geometry from a file and add to the scene
    myInput = SoInput()
    if not myInput.openFile("dogDish.iv"):
        sys.exit(1)
    geomObject = SoDB.readAll(myInput)
    if geomObject == None:
        sys.exit(1)
    root.addChild(geomObject)
   
    # Make the scene graph visible
    myRenderArea.setSceneGraph(root)
   
    # Attach the material editor to the material in the scene
    myEditor.attach(myMaterial)
   
    # Show the main window
    myRenderArea.show()
    myEditor.show()
    SoGui.show(myForm)    # this calls QtManageChild
    SoGui.show(myWindow)  # this calls QtRealizeWidget
   
    # Loop forever
    SoGui.mainLoop()

if __name__ == "__main__":
    print("This example is not functional as it is GUI toolkit dependent!")
    sys.exit(1)
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 6, example 2.
#
# This example renders a globe and uses 3D text to label the
# continents Africa and Asia.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoGroup()

    # Choose a font
    myFont = SoFont()
    myFont.name = "Times-Roman"
    myFont.size = .2
    root.addChild(myFont)

    # We'll color the front of the text white, and the sides 
    # dark grey. So use a materialBinding of PER_PART and
    # two diffuseColor values in the material node.
    myMaterial = SoMaterial()
    myBinding = SoMaterialBinding()
    myMaterial.diffuseColor.set1Value(0, SbColor(1,1,1))
    myMaterial.diffuseColor.set1Value(1, SbColor(.1,.1,.1))
    myBinding.value = SoMaterialBinding.PER_PART
    root.addChild(myMaterial)
    root.addChild(myBinding)

    # Create the globe
    sphereSep = SoSeparator()
    myTexture2 = SoTexture2()
    sphereComplexity = SoComplexity()
    sphereComplexity.value = 0.55
    root.addChild(sphereSep)
    sphereSep.addChild(myTexture2)
    sphereSep.addChild(sphereComplexity)
    sphereSep.addChild(SoSphere())
    myTexture2.filename = "globe.rgb"

    # Add Text3 for AFRICA, transformed to proper location.
    africaSep = SoSeparator()
    africaTransform = SoTransform()
    africaText = SoText3()
    africaTransform.rotation.setValue(SbVec3f(0,1,0), .4)
    africaTransform.translation = (.25, .0, 1.25)
    africaText.parts = SoText3.ALL
    africaText.string = "AFRICA"
    root.addChild(africaSep)
    africaSep.addChild(africaTransform)
    africaSep.addChild(africaText)

    # Add Text3 for ASIA, transformed to proper location.
    asiaSep = SoSeparator()
    asiaTransform = SoTransform()
    asiaText = SoText3()
    asiaTransform.rotation.setValue(SbVec3f(0,1,0), 1.5)
    asiaTransform.translation = (.8, .6, .5)
    asiaText.parts = SoText3.ALL
    asiaText.string = "ASIA"
    root.addChild(asiaSep)
    asiaSep.addChild(asiaTransform)
    asiaSep.addChild(asiaText)

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("3D Text")

    # In Inventor 2.1, if the machine does not have hardware texture
    # mapping, we must override the default drawStyle to display textures.
    myViewer.setDrawStyle(SoGuiViewer.STILL, SoGuiViewer.VIEW_AS_IS)

    myViewer.show()
    myViewer.viewAll()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 12, example 4.
#
# Timer sensors.  An object is rotated by a timer sensor.
# (called "rotatingSensor").  The interval between calls 
# controls how often it rotates.
# A second timer (called "schedulingSensor") goes off
# every 5 seconds and changes the interval of 
# "rotatingSensor".  The interval alternates between
# once per second and 10 times per second.
# This example could also be done using engines.
#

import sys

from pivy.coin import *
from pivy.sogui import *

###########################################################
# CODE FOR The Inventor Mentor STARTS HERE

# This function is called either 10 times/second or once every
# second the scheduling changes every 5 seconds (see below):
def rotatingSensorCallback(myRotation, sensor):
    # Rotate an object...
    currentRotation = myRotation.rotation.getValue()
    currentRotation *= SbRotation(SbVec3f(0,0,1), M_PI/90.0)
    myRotation.rotation.setValue(currentRotation)

# This function is called once every 5 seconds, and
# reschedules the other sensor.
def schedulingSensorCallback(rotatingSensor, sensor):
    rotatingSensor.unschedule()
    if rotatingSensor.getInterval() == 1.0:
        rotatingSensor.setInterval(1.0/10.0)
    else: rotatingSensor.setInterval(1.0)
    rotatingSensor.schedule()

# CODE FOR The Inventor Mentor ENDS HERE
###########################################################

def main():
    if len(sys.argv) != 2:
        print("Usage: %s filename.iv" % (sys.argv[0]), file=sys.stderr)
        sys.exit(1)

    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoSeparator()
   
###########################################################   
# CODE FOR The Inventor Mentor STARTS HERE

    myRotation = SoRotation()
    root.addChild(myRotation)

    rotatingSensor = SoTimerSensor(rotatingSensorCallback, myRotation)
    rotatingSensor.setInterval(1.0) # scheduled once per second
    rotatingSensor.schedule()

    schedulingSensor = SoTimerSensor(schedulingSensorCallback, rotatingSensor)
    schedulingSensor.setInterval(5.0) # once per 5 seconds
    schedulingSensor.schedule()

# CODE FOR The Inventor Mentor ENDS HERE
###########################################################

    inputFile = SoInput()
    if inputFile.openFile(sys.argv[1]) == 0:
        print("Could not open file %s" % (sys.argv[1]), sys.stderr)
        sys.exit(1)
        
    root.addChild(SoDB.readAll(inputFile))

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Two Timers")
    myViewer.show()

    SoGui.show(myWindow)  # Display main window
    SoGui.mainLoop()        # Main Inventor event loop

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8

###
# Copyright (c) 2005 Øystein Handegard <handegar@sim.no>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

from __future__ import print_function
import math
from pivy.coin import *
from pivy.sogui import *

##############################################################

def generateColoredLights(distance):
    scenegraph = SoSeparator()
    rotor = SoRotor()
    rotor.rotation = (1, 1, 1, 0)
    rotor.speed = 0.2

    redlight = SoPointLight()
    redlight.color = (1, 0, 0)
    redlight.location = (0, distance, 0)
    
    goldlight = SoPointLight()
    goldlight.color = (1, 185.0/256, 75.0/256)
    goldlight.location = (distance, 0, 0)
    
    trans = SoTranslation()
    trans.translation = (distance, 0, distance)

    scenegraph.addChild(rotor)
    scenegraph.addChild(redlight)
    scenegraph.addChild(rotor)
    scenegraph.addChild(goldlight)
    return scenegraph
    
def generateTile(height, width, thickness):
    h = height*(1-(8*thickness))
    w = width*(1-(8*thickness))
    s = (8*thickness)+1.0
    t = -thickness

    points = ((0, -w/2.0, -h/2.0), (0, w/2.0, -h/2.0), (0, w/2.0, h/2.0), (0, -w/2.0, h/2.0),
              (t, -(s*w)/2.0, -(s*h)/2.0), (t, (s*w)/2.0, (-s*h)/2.0), (t, (s*w)/2.0, (s*h)/2.0), (t, (-s*w)/2.0, (s*h)/2.0))
    indices = (0, 1, 2, 3, -1,  4, 5, 1, 0, -1,  5, 6, 2, 1, -1,  3, 2, 6, 7, -1,  0, 3, 7, 4, -1 )

    coordset = SoCoordinate3()
    coordset.point.setValues(0, len(points), points)

    faceset = SoIndexedFaceSet()
    faceset.coordIndex.setValues(0, len(indices), indices)

    sep = SoSeparator()
    sep.addChild(coordset)
    sep.addChild(faceset)
    
    return sep
    

def generateMirrorBall(radius, tileheight, tilewidth):
    root = SoSeparator()
    
    tilethickness = ((tilewidth + tileheight)/2.0)/50.0 # looks nice
    tile = generateTile(tileheight, tilewidth, tilethickness)
        
    radiustrans = SoTranslation()
    radiustrans.translation = SbVec3f(radius,0,0)

    mirrormat = SoMaterial()
    mirrormat.specularColor = (1, 1, 1)
    mirrormat.diffuseColor = (0.2, 0.2, 0.2)    
    mirrormat.shinyness = 0.5
    root.addChild(mirrormat)

    rot1 = SbRotation()
    rot2 = SbRotation() 

    x = M_PI
    anglesteplong = M_PI / (2.0 * math.asin(tileheight / (2.0 * radius)))
    anglesteplat = M_PI / (2.0 * math.asin(tilewidth / (2.0 * radius)))

    while x > 0:
        y = M_PI*2
        while y > 0:
            sep = SoSeparator()
            rot1.setValue(SbVec3f(0,0,1), x)
            rot2.setValue(SbVec3f(1, 0, 0), y)
            y = y - (M_PI*2 / ((anglesteplong*2)*math.sin(x)))            
            rot = SoRotation()
            rot.rotation = (rot1*rot2)
            sep.addChild(rot)
            sep.addChild(radiustrans)
            sep.addChild(tile)
            root.addChild(sep)
        x = x - (M_PI / anglesteplat)

    return root
    
##############################################################

def main():
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)
    
    # Radius and tile size of the mirror-ball
    radius = 8
    tilesizex = 1
    tilesizey = 0.8

    # Create rotating lights around the ball
    lights = generateColoredLights(radius * 50.0)
    lights.addChild(SoResetTransform())

    # Generate mirror ball.
    print("Generating a mirror ball with radius %d and tile size %.1fx%.1f..." % (radius, tilesizex, tilesizey))
    lights.addChild(generateMirrorBall(radius, tilesizex, tilesizey))
    print("...finished.")

    # setup viewer
    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(lights)
    myViewer.setTitle("Examiner Viewer")
    myViewer.viewAll()
    myViewer.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

    return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor
# chapter 7, example 2.
#
# This example illustrates using texture coordindates on
# a Face Set.
#

import sys

from pivy.coin import *
from pivy.sogui import *

# set this variable to 0 if you want to use the other method
IV_STRICT = 1

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoSeparator()
    
    # Choose a texture
    brick = SoTexture2()
    root.addChild(brick)
    brick.filename = "brick.1.rgb"

    if IV_STRICT:
        # This is the preferred code for Inventor 2.1 

        # Using the new SoVertexProperty node is more efficient
        myVertexProperty = SoVertexProperty()

        # Define the square's spatial coordinates
        myVertexProperty.vertex.set1Value(0, SbVec3f(-3, -3, 0))
        myVertexProperty.vertex.set1Value(1, SbVec3f( 3, -3, 0))
        myVertexProperty.vertex.set1Value(2, SbVec3f( 3,  3, 0))
        myVertexProperty.vertex.set1Value(3, SbVec3f(-3,  3, 0))

        # Define the square's normal
        myVertexProperty.normal.set1Value(0, SbVec3f(0, 0, 1))

        # Define the square's texture coordinates
        myVertexProperty.texCoord.set1Value(0, SbVec2f(0, 0))
        myVertexProperty.texCoord.set1Value(1, SbVec2f(1, 0))
        myVertexProperty.texCoord.set1Value(2, SbVec2f(1, 1))
        myVertexProperty.texCoord.set1Value(3, SbVec2f(0, 1))

        # SoTextureCoordinateBinding node is now obsolete--in Inventor 2.1,
        # texture coordinates will always be generated if none are 
        # provided.
        #
        # tBind = SoTextureCoordinateBinding()
        # root.addChild(tBind)
        # tBind.value(SoTextureCoordinateBinding.PER_VERTEX)
        #
        # Define normal binding
        myVertexProperty.normalBinding = SoNormalBinding.OVERALL

        # Define a FaceSet
        myFaceSet = SoFaceSet()
        root.addChild(myFaceSet)
        myFaceSet.numVertices.set1Value(0, 4)

        myFaceSet.vertexProperty.setValue(myVertexProperty)

    else:
        # Define the square's spatial coordinates
        coord = SoCoordinate3()
        root.addChild(coord)
        coord.point.set1Value(0, SbVec3f(-3, -3, 0))
        coord.point.set1Value(1, SbVec3f( 3, -3, 0))
        coord.point.set1Value(2, SbVec3f( 3,  3, 0))
        coord.point.set1Value(3, SbVec3f(-3,  3, 0))

        # Define the square's normal
        normal = SoNormal()
        root.addChild(normal)
        normal.vector.set1Value(0, SbVec3f(0, 0, 1))

        # Define the square's texture coordinates
        texCoord = SoTextureCoordinate2()
        root.addChild(texCoord)
        texCoord.point.set1Value(0, SbVec2f(0, 0))
        texCoord.point.set1Value(1, SbVec2f(1, 0))
        texCoord.point.set1Value(2, SbVec2f(1, 1))
        texCoord.point.set1Value(3, SbVec2f(0, 1))

        # Define normal binding
        nBind = SoNormalBinding()
        root.addChild(nBind)
        nBind.value = SoNormalBinding.OVERALL

        # SoTextureCoordinateBinding node is now obsolete--in Inventor 2.1,
        # texture coordinates will always be generated if none are 
        # provided.
        #
        # tBind = SoTextureCoordinateBinding()
        # root.addChild(tBind)
        # tBind.value.setValue(SoTextureCoordinateBinding.PER_VERTEX)
        #

        # Define a FaceSet
        myFaceSet = SoFaceSet()
        root.addChild(myFaceSet)
        myFaceSet.numVertices.set1Value(0, 4)

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Texture Coordinates")

    # In Inventor 2.1, if the machine does not have hardware texture
    # mapping, we must override the default drawStyle to display textures.
    # myViewer.setDrawStyle(SoGuiViewer.STILL, SoGuiViewer.VIEW_AS_IS)

    myViewer.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 13, example 8.
#
# Rotor node example.  
# Read in the tower and vanes of a windmill from a file.
# Use a rotor node to rotate the vanes.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def readFile(filename):
    # Open the input file
    mySceneInput = SoInput()
    if not mySceneInput.openFile(filename):
        print("Cannot open file %s" % (filename), file=sys.stderr)
        return None

    # Read the whole file into the database
    myGraph = SoDB.readAll(mySceneInput)
    if myGraph == None:
        print("Problem reading file", file=sys.stderr)
        return None

    mySceneInput.closeFile()
    return myGraph

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])  
    if myWindow == None: sys.exit(1)     

    root = SoSeparator()

    # Read in the data for the windmill tower
    windmillTower = readFile("windmillTower.iv")
    root.addChild(windmillTower)

    # Add a rotor node to spin the vanes
    myRotor = SoRotor()
    myRotor.rotation.setValue(SbVec3f(0, 0, 1), 0) # z axis
    myRotor.speed = 0.2
    root.addChild(myRotor)

    # Read in the data for the windmill vanes
    windmillVanes = readFile("windmillVanes.iv")
    root.addChild(windmillVanes)

    # Create a viewer
    myViewer = SoGuiExaminerViewer(myWindow)

    # attach and show viewer
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Windmill")
    myViewer.show()

    SoDB.setRealTimeInterval(1/120.0)
    
    # Loop forever
    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



from __future__ import print_function
import sys
from pivy import coin
from pivy.gui import soqt
# import shiboken if you want to use the widget within qt

# Initialize Coin (returns main window to use)
# If unsuccessful, exit.
myWindow = soqt.SoQt.init(sys.argv[0])
print(myWindow)
# Make a scene containing a red cone.
myMaterial = coin.SoMaterial()
myMaterial.diffuseColor = (1.0, 0.0, 0.0)
scene = coin.SoSeparator() 
scene.ref()
scene.addChild(myMaterial)
scene.addChild(coin.SoCone())
# Create a viewer to visualize our scene.
viewer = soqt.SoQtExaminerViewer(myWindow)
# Put our scene into viewer, change the title.
viewer.setSceneGraph(scene)
viewer.setTitle("Examiner Viewer")
viewer.show()
soqt.SoQt.show(myWindow)
soqt.SoQt.mainLoop()


#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from The Inventor Mentor
# chapter 17, example 1.
#
# This examples shows how the user can create a custom
# X visual for doing color index rendering with
# an Inventor Viewer. This shows how to create the right
# visual, as well as load the color map with the wanted
# colors.
#

from __future__ import print_function
import sys

from OpenGL.GLUT import *
from OpenGL.GL import *
from OpenGL.GLX import *

from pivy.coin import *
from pivy.sogui import *

# window attribute list to create a color index visual.
# This will create a double buffered color index window
# with the maximum number of bits and a zbuffer.
GLX_DOUBLEBUFFER=5
GLX_BUFFER_SIZE=2
GLX_DEPTH_SIZE=12
attribList = (GLX_DOUBLEBUFFER, 
              GLX_BUFFER_SIZE, 1, 
              GLX_DEPTH_SIZE, 1, 
              None)

# list of colors to load in the color map
colors = ((.2, .2, .2), (.5, 1, .5), (.5, .5, 1))

sceneBuffer = """#Inventor V2.0 ascii

Separator {
   LightModel { model BASE_COLOR }
   ColorIndex { index 1 }
   Coordinate3 { point [ -1 -1 -1, -1 1 -1, 1 1 1, 1 -1 1] }
   FaceSet {}
   ColorIndex { index 2 }
   Coordinate3 { point [ -1 -1 1, -1 1 1, 1 1 -1, 1 -1 -1] }
   FaceSet {}
}"""

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
   
    # read the scene graph in
    input = SoInput()
    input.setBuffer(sceneBuffer)
    scene = SoDB.readAll(input) 
    if not scene:
        print("Couldn't read scene")
        sys.exit(1)

    # create the color index visual
    vis = glXChooseVisual(QtDisplay(myWindow), 
                          XScreenNumberOfScreen(QtScreen(myWindow)),
                          attribList)
    if not vis:
        print("Couldn't create visual")
        sys.exit(1)
   
    # allocate the viewer, set the scene, the visual and
    # load the color map with the wanted colors.
    #
    # Color 0 will be used for the background (default) while
    # color 1 and 2 are used by the objects.
    #
    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setNormalVisual(vis)
    myViewer.setColorMap(0, 3, colors)
    myViewer.setSceneGraph(scene)
    myViewer.setTitle("Color Index Mode")
   
    # Show the viewer and loop forever...
    myViewer.show()
    XtRealizeWidget(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    print("This example is not functional as it is GLX dependent!")
    sys.exit(1)
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 12, example 2.
#
# Using getTriggerNode/getTriggerField methods of the data
# sensor.
#

from __future__ import print_function
import sys

from pivy.coin import *

# Sensor callback function:
def rootChangedCB(void, mySensor):
    # Sensors get autocasted; there is no need to cast them manually
    # through the cast() function, such as:
    #   mySensor = cast(s, "SoDataSensor")
    # mySensor is therefore a SoNodeSensor.

    changedNode = mySensor.getTriggerNode()
    changedField = mySensor.getTriggerField()
    
    print("The node named '%s' changed" % (changedNode.getName().getString()))

    if changedField:
        # the pythonic getFieldName() method returns a string or None in Pivy.
        fieldName = changedNode.getFieldName(changedField)
        print(" (field %s)" % (fieldName))
    else:
        print(" (no fields changed)")

def main():
    # SoDB.init() invoked automatically upon coin module import

    root = SoSeparator()
    root.setName("Root")

    myCube = SoCube()
    root.addChild(myCube)
    myCube.setName("MyCube")

    mySphere = SoSphere()
    root.addChild(mySphere)
    mySphere.setName("MySphere")

    mySensor = SoNodeSensor(rootChangedCB, None)
    mySensor.setPriority(0)
    # mySensor.setFunction(rootChangedCB)
    mySensor.attach(root)

    # Now, make a few changes to the scene graph the sensor's
    # callback function will be called immediately after each
    # change.
    myCube.width = 1.0
    myCube.height = 2.0
    mySphere.radius = 3.0
    root.removeChild(mySphere)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor.
# chapter 14, example 1.
#
# Use SoShapeKits to create two 3-D words, "NICE" and "HAPPY"
# Use nodekit methods to access the fields of the "material"
# and "transform" parts.
# Use a calculator engine and an elapsed time engine to make
# the words change color and fly about the screen.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])  
    if myWindow == None: sys.exit(1)     

    root = SoSeparator()

    # Create shape kits with the words "HAPPY" and "NICE"
    happyKit = SoShapeKit()
    root.addChild(happyKit)
    happyKit.setPart("shape", SoText3())
    happyKit.set("shape { parts ALL string \"HAPPY\"}")
    happyKit.set("font { size 2}")

    niceKit = SoShapeKit()
    root.addChild(niceKit)
    niceKit.setPart("shape", SoText3())
    niceKit.set("shape { parts ALL string \"NICE\"}")
    niceKit.set("font { size 2}")

    # Create the Elapsed Time engine
    myTimer = SoElapsedTime()

    # Create two calculator - one for HAPPY, one for NICE.
    happyCalc = SoCalculator()
    happyCalc.a.connectFrom(myTimer.timeOut)
    happyCalc.expression = """ta=cos(2*a); tb=sin(2*a);
                              oA = vec3f(3*pow(ta,3),3*pow(tb,3),1);
                              oB = vec3f(fabs(ta)+.1,fabs(.5*fabs(tb))+.1,1);
                              oC = vec3f(fabs(ta),fabs(tb),.5)"""

    # The second calculator uses different arguments to
    # sin() and cos(), so it moves out of phase.
    niceCalc = SoCalculator()
    niceCalc.a.connectFrom(myTimer.timeOut)
    niceCalc.expression = """ta=cos(2*a+2); tb=sin(2*a+2);
                             oA = vec3f(3*pow(ta,3),3*pow(tb,3),1);
                             oB = vec3f(fabs(ta)+.1,fabs(.5*fabs(tb))+.1,1);
                             oC = vec3f(fabs(ta),fabs(tb),.5)"""

    # Connect the transforms from the calculators...
    happyXf = happyKit.getPart("transform",TRUE)
    happyXf.translation.connectFrom(happyCalc.oA)
    happyXf.scaleFactor.connectFrom(happyCalc.oB)
    niceXf = niceKit.getPart("transform",TRUE)
    niceXf.translation.connectFrom(niceCalc.oA)
    niceXf.scaleFactor.connectFrom(niceCalc.oB)

    # Connect the materials from the calculators...
    happyMtl = happyKit.getPart("material",TRUE)
    happyMtl.diffuseColor.connectFrom(happyCalc.oC)
    niceMtl = niceKit.getPart("material",TRUE)
    niceMtl.diffuseColor.connectFrom(niceCalc.oC)

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Frolicking Words")
    myViewer.viewAll()
    myViewer.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 6, example 1.
#
# This example renders a globe and uses 2D text to label the
# continents Africa and Asia.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoGroup()

    # Choose a font
    myFont = SoFont()
    myFont.name = "Times-Roman"
    myFont.size = 24.0
    root.addChild(myFont)

    # Add the globe, a sphere with a texture map.
    # Put it within a separator.
    sphereSep = SoSeparator()
    myTexture2 = SoTexture2()
    sphereComplexity = SoComplexity()
    sphereComplexity.value = 0.55
    root.addChild(sphereSep)
    sphereSep.addChild(myTexture2)
    sphereSep.addChild(sphereComplexity)
    sphereSep.addChild(SoSphere())
    myTexture2.filename = "globe.rgb"

    # Add Text2 for AFRICA, translated to proper location.
    africaSep = SoSeparator()
    africaTranslate = SoTranslation()
    africaText = SoText2()
    africaTranslate.translation = (.25,.0,1.25)
    africaText.string = "AFRICA"
    root.addChild(africaSep)
    africaSep.addChild(africaTranslate)
    africaSep.addChild(africaText)

    # Add Text2 for ASIA, translated to proper location.
    asiaSep = SoSeparator()
    asiaTranslate = SoTranslation()
    asiaText = SoText2()
    asiaTranslate.translation = (.8,.8,0)
    asiaText.string = "ASIA"
    root.addChild(asiaSep)
    asiaSep.addChild(asiaTranslate)
    asiaSep.addChild(asiaText)

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("2D Text")

    # In Inventor 2.1, if the machine does not have hardware texture
    # mapping, we must override the default drawStyle to display textures.
    myViewer.setDrawStyle(SoGuiViewer.STILL, SoGuiViewer.VIEW_AS_IS)

    myViewer.show()
    myViewer.viewAll()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from The Inventor Mentor,
# chapter 10, example 6.
#
# This example demonstrates the use of the pick filter
# callback to implement a top level selection policy.
# That is, always select the top most group beneath the
# selection node,  rather than selecting the actual
# shape that was picked.
#

from __future__ import print_function
import sys

from pivy.coin import *
from pivy.sogui import *

# Pick the topmost node beneath the selection node
def pickFilterCB(void, pick):
    # See which child of selection got picked
    p = pick.getPath()
    
    for i in range(p.getLength() - 1):
        n = p.getNode(i)
        if n.isOfType(SoSelection.getClassTypeId()):
            break

    # Copy 2 nodes from the path:
    # selection and the picked child
    return p.copy(i, 2)

def main():
    # Initialization
    mainWindow = SoGui.init(sys.argv[0])
    
    # Open the data file
    input = SoInput()
    datafile = "parkbench.iv"
    if not input.openFile(datafile):
        print("Cannot open %s for reading." % (datafile), file=sys.stderr)
        sys.exit(1)

    # Read the input file
    sep = SoSeparator()
    sep.addChild(SoDB.readAll(input))
   
    # Create two selection roots - one will use the pick filter.
    topLevelSel = SoSelection()
    topLevelSel.addChild(sep)
    topLevelSel.setPickFilterCallback(pickFilterCB)

    defaultSel = SoSelection()
    defaultSel.addChild(sep)

    # Create two viewers, one to show the pick filter for top level
    # selection, the other to show default selection.
    viewer1 = SoGuiExaminerViewer(mainWindow)
    viewer1.setSceneGraph(topLevelSel)
    boxhra1 = SoBoxHighlightRenderAction()
    viewer1.setGLRenderAction(boxhra1)
    viewer1.redrawOnSelectionChange(topLevelSel)
    viewer1.setTitle("Top Level Selection")

    viewer2 = SoGuiExaminerViewer()
    viewer2.setSceneGraph(defaultSel)
    boxhra2 = SoBoxHighlightRenderAction()
    viewer2.setGLRenderAction(boxhra2)
    viewer2.redrawOnSelectionChange(defaultSel)
    viewer2.setTitle("Default Selection")

    viewer1.show()
    viewer2.show()
   
    SoGui.show(mainWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



import sys
from PySide2.QtGui import QColor
from PySide2.QtWidgets import QApplication
from pivy import quarter, coin


# testing if a eventcallback can remove itself.

class test(coin.SoSeparator):
    def __init__(self):
        super(test, self).__init__()
        self.events = coin.SoEventCallback()
        self += self.events
        self.cb = self.events.addEventCallback(
            coin.SoLocation2Event.getClassTypeId(), self.my_cb)
        self.cb1 = self.events.addEventCallback(
            coin.SoEvent.getClassTypeId(), self.my_cb_1)

    def my_cb(self, *args):
        self.events.removeEventCallback(
            coin.SoLocation2Event.getClassTypeId(), self.cb)

    def my_cb_1(self, *args):
        self.events.removeEventCallback(
            coin.SoEvent.getClassTypeId(), self.cb1)


def main():
    app = QApplication(sys.argv)
    viewer = quarter.QuarterWidget()

    root = coin.SoSeparator()
    root += coin.SoCone()
    root += test()

    viewer.setSceneGraph(root)
    viewer.setBackgroundColor(QColor(255, 255, 255))
    viewer.setWindowTitle("minimal")
    viewer.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from The Inventor Mentor,
# chapter 10, example 8.
#
# This example demonstrates the use of the pick filter
# callback to always select nodekits. This makes it especially
# easy to edit attributes of objects because the nodekit takes
# care of the part creation details.
#

from __future__ import print_function
import math, sys

from pivy.coin import *
from pivy.sogui import *

class UserData:
    sel = None
    editor = None
    ignore = None

##############################################################
## CODE FOR The Inventor Mentor STARTS HERE  (part 1)

# Truncate the pick path so a nodekit is selected
def pickFilterCB(void, pick):
    # See which child of selection got picked
    p = pick.getPath()

    for i in range(p.getLength() - 1, -1, -1):
        n = p.getNode(i)
        if n.isOfType(SoShapeKit.getClassTypeId()):
            break

    # Copy the path down to the nodekit
    return p.copy(0, i+1)

# CODE FOR The Inventor Mentor ENDS HERE
##############################################################


# Create a sample scene graph
def buildScene():
    g = SoGroup()
    
    # Place a dozen shapes in circular formation
    for i in range(12):
        k = SoShapeKit()
        k.setPart("shape", SoCube())
        xf = k.getPart("transform", TRUE)
        xf.translation = (8*math.sin(i*M_PI/6), 8*math.cos(i*M_PI/6), 0.0)
        g.addChild(k)

    return g

# Update the material editor to reflect the selected object
def selectCB(userData, path):
    kit = path.getTail()
    kitMtl = kit.getPart("material", TRUE)

    # ud = userData
    userData.ignore = TRUE
    userData.editor.setMaterial(kitMtl)
    userData.ignore = FALSE

# This is called when the user chooses a new material
# in the material editor. This updates the material
# part of each selected node kit.
def mtlChangeCB(userData, mtl):
    # Our material change callback is invoked when the
    # user changes the material, and when we change it
    # through a call to SoGuiMaterialEditor.setMaterial.
    # In this latter case, we ignore the callback invocation
    # ud = userData

    if userData.ignore:
        return

    sel = userData.sel
    
    # Our pick filter guarantees the path tail will
    # be a shape kit.
    for i in range(sel.getNumSelected()):
        p = sel.getPath(i)
        kit = p.getTail()
        kitMtl = kit.getPart("material", TRUE)
        kitMtl.copyFieldValues(mtl)

def main():
    # Initialization
    mainWindow = SoGui.init(sys.argv[0])
    
    # Create our scene graph.
    sel = SoSelection()
    sel.addChild(buildScene())

    # Create a viewer with a render action that displays highlights
    viewer = SoGuiExaminerViewer(mainWindow)
    viewer.setSceneGraph(sel)
    boxhra = SoBoxHighlightRenderAction()
    viewer.setGLRenderAction(boxhra)
    viewer.redrawOnSelectionChange(sel)
    viewer.setTitle("Select Node Kits")
    viewer.show()

    # Create a material editor
    try:
        ed = SoGuiMaterialEditor()
    except:
        print("The SoGuiMaterialEditor node has not been implemented in the " + \
              "SoGui bindings of Coin!")
        sys.exit(1)
    ed.show()

    # User data for our callbacks
    userData = UserData()
    userData.sel = sel
    userData.editor = ed
    userData.ignore = FALSE
   
    # Selection and material change callbacks
    ed.addMaterialChangedCallback(mtlChangeCB, userData)
    sel.setPickFilterCallback(pickFilterCB)
    sel.addSelectionCallback(selectCB, userData)
   
    SoGui.show(mainWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor Programming Guide,
# chapter 8, example 4.
#
# This example creates and displays a Bezier Surface
# with trim curves.  The surface is identical to the
# surface in example 08.3.BezSurf.  The SoNurbsProfile
# class is used to define the trims.
# 
# One trim curve makes a counter-clockwise square around
# the whole surface.  Two other trim curves are combined
# end to end to cut a hole from the surface.  The outside
# trim curve and the first inside trim curve are both
# order 2 curves and are therefore sets of straight lines
# in parameter space.  The second inside curve is a
# Bezier curve.
#

import sys

from pivy.coin import *
from pivy.sogui import *

floorData = """#Inventor V2.0 ascii
Separator {
   SpotLight {
      cutOffAngle 0.9
      dropOffRate 0.2 
      location 6 12 2 
      direction 0 -1 0
   }
   ShapeHints {
      faceType UNKNOWN_FACE_TYPE
   }
   Texture2Transform {
      #rotation 1.57
      scaleFactor 8 8
   }
   Texture2 {
      filename oak.rgb
   }
   NormalBinding {
        value  PER_PART
   }
   Material { diffuseColor 1 1 1 specularColor 1 1 1 shininess 0.4 }
   DEF FloorPanel Separator {
      DEF FloorStrip Separator {
         DEF FloorBoard Separator {
            Normal { vector 0 1 0 }
            TextureCoordinate2 {
               point [ 0 0, 0.5 0, 0.5 2, 0.5 4, 0.5 6,
                       0.5 8, 0 8, 0 6, 0 4, 0 2 ] }
            Coordinate3 {
               point [ 0 0 0, .5 0 0, .5 0 -2, .5 0 -4, .5 0 -6,
                       .5 0 -8, 0 0 -8, 0 0 -6, 0 0 -4, 0 0 -2, ]
            }
            FaceSet { numVertices 10 }
            BaseColor { rgb 0.3 0.1 0.0 }
            Translation { translation 0.125 0 -0.333 }
            Cylinder { parts TOP radius 0.04167 height 0.002 }
            Translation { translation 0.25 0 0 }
            Cylinder { parts TOP radius 0.04167 height 0.002 }
            Translation { translation 0 0 -7.333 }
            Cylinder { parts TOP radius 0.04167 height 0.002 }
            Translation { translation -0.25 0 0 }
            Cylinder { parts TOP radius 0.04167 height 0.002 }
         }
         Translation { translation 0 0 8.03 }
         USE FloorBoard
         Translation { translation 0 0 8.04 }
         USE FloorBoard
      }
      Translation { translation 0.53 0 -0.87 }
      USE FloorStrip
      Translation { translation 0.53 0 -2.3 }
      USE FloorStrip
      Translation { translation 0.53 0 1.3 }
      USE FloorStrip
      Translation { translation 0.53 0 1.1 }
      USE FloorStrip
      Translation { translation 0.53 0 -0.87 }
      USE FloorStrip
      Translation { translation 0.53 0 1.7 }
      USE FloorStrip
      Translation { translation 0.53 0 -0.5 }
      USE FloorStrip
   }
   Translation { translation 4.24 0 0 }
   USE FloorPanel
   Translation { translation 4.24 0 0 }
   USE FloorPanel
}"""

############################################################
# CODE FOR The Inventor Mentor STARTS HERE

# The array of trim coordinates
tpts = (
   (0.0, 0.0),
   (1.0, 0.0),
   (1.0, 1.0),
   (0.0, 1.0),
   (0.2, 0.2),
   (0.2, 0.7),
   (0.9, 0.7),
   (0.9, 0.2),
   (0.7, 0.0),
   (0.4, 0.8))

# The 16 coordinates defining the Bezier surface. 
pts = (
   (-4.5, -2.0,  8.0),
   (-2.0,  1.0,  8.0),
   ( 2.0, -3.0,  6.0),
   ( 5.0, -1.0,  8.0),
   (-3.0,  3.0,  4.0),
   ( 0.0, -1.0,  4.0),
   ( 1.0, -1.0,  4.0),
   ( 3.0,  2.0,  4.0),
   (-5.0, -2.0, -2.0),
   (-2.0, -4.0, -2.0),
   ( 2.0, -1.0, -2.0),
   ( 5.0,  0.0, -2.0),
   (-4.5,  2.0, -6.0),
   (-2.0, -4.0, -5.0),
   ( 2.0,  3.0, -5.0),
   ( 4.5, -2.0, -6.0))

# The 3 knot vectors for the 3 trim curves.
tknots1 = (0, 0, 1, 2, 3, 4, 4)
tknots2 = (0, 0, 1, 2, 3, 3)
tknots3 = (0, 0, 0, 0, 1, 1, 1, 1)

# The Bezier knot vector for the surface.
# This knot vector is used in both the U and
# V directions.
knots = (0, 0, 0, 0, 1, 1, 1, 1)

# Create the nodes needed for the Bezier patch
# and its trim curves.
def makeSurface():
    surfSep = SoSeparator()

    # Define the Bezier surface including the control
    # points, trim curve, and a complexity.
    complexity = SoComplexity()
    controlPts = SoCoordinate3()
    surface    = SoNurbsSurface()
    complexity.value = 0.7
    controlPts.point.setValues(0, 16, pts)
    surface.numUControlPoints = 4
    surface.numVControlPoints = 4
    surface.uKnotVector.setValues(0, 8, knots)
    surface.vKnotVector.setValues(0, 8, knots)
    surfSep.addChild(complexity)
    surfSep.addChild(controlPts)
    
    trimPts = SoProfileCoordinate2()
    nTrim1 = SoNurbsProfile()
    nTrim2 = SoNurbsProfile()
    nTrim3 = SoNurbsProfile()

    trimPts.point.setValues(0, 12, tpts)
    trimInds = (0, 1, 2, 3, 0)
    nTrim1.index.setValues(0, 5, trimInds)
    nTrim1.knotVector.setValues(0, 7, tknots1)
    trimInds = (4, 5, 6, 7)
    nTrim2.linkage = SoProfile.START_NEW
    nTrim2.index.setValues(0, 4, trimInds)
    nTrim2.knotVector.setValues(0, 6, tknots2)
    trimInds = (7, 8, 9, 4)
    nTrim3.linkage = SoProfile.ADD_TO_CURRENT
    nTrim3.index.setValues(0, 4, trimInds)
    nTrim3.knotVector.setValues(0, 8, tknots3)

    surfSep.addChild(trimPts)
    surfSep.addChild(nTrim1)
    surfSep.addChild(nTrim2)
    surfSep.addChild(nTrim3)
    surfSep.addChild(surface)

    return surfSep

# CODE FOR The Inventor Mentor ENDS HERE
############################################################


def main():
    # Initialize Inventor and Qt
    appWindow = SoGui.init(sys.argv[0])
    if appWindow == None:
        sys.exit(1)

    root = SoSeparator()

    rot = SoRotation()
    rot.rotation.setValue(SbRotation(SbVec3f(0.0, 1.0, 0.0), M_PI/2.0))
    root.addChild(rot)

    # Create the scene graph for the carpet
    carpet = SoSeparator()
    surf   = makeSurface()
    tex    = SoTexture2()

    tex.filename = "diamondRug.rgb"
    carpet.addChild(tex)
    carpet.addChild(surf)
    root.addChild(carpet)

    # Create the scene graph for the floor
    floor = SoSeparator()
    xlate = SoTranslation()
    scale = SoScale()
    input = SoInput()

    input.setBuffer(floorData)
    result = SoDB.readAll(input)
    xlate.translation = (-12.0, -5.0, -5.0)
    scale.scaleFactor = (2.0, 1.0, 2.0)
    floor.addChild(xlate)
    floor.addChild(scale)
    floor.addChild(result)
    root.addChild(floor)

    # Create the scene graph for the carpet's shadow
    shadow = SoSeparator()
    shmdl  = SoLightModel()
    shmtl  = SoMaterial()
    shclr  = SoBaseColor()
    shxl   = SoTranslation()
    shscl  = SoScale()

    shmdl.model = SoLightModel.BASE_COLOR
    shclr.rgb = (0.21, 0.15, 0.09)
    shmtl.transparency = 0.3
    shxl.translation = (0.0, -4.9, 0.0)
    shscl.scaleFactor = (1.0, 0.0, 1.0)
    shadow.addChild(shmtl)
    shadow.addChild(shmdl)
    shadow.addChild(shclr)
    shadow.addChild(shxl)
    shadow.addChild(shscl)
    shadow.addChild(surf)
    root.addChild(shadow)

    # Initialize an Examiner Viewer
    viewer = SoGuiExaminerViewer(appWindow)
    viewer.setSceneGraph(root)
    viewer.setTitle("Trimmed Nurbs Surface")
    cam = viewer.getCamera()
    cam.position = (-8.5, 13.0, 23.0)
    cam.pointAt(SbVec3f(-2.0, -2.0, -4.0))
    viewer.show()

    SoGui.show(appWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8

###
# Copyright (c) 2005 Øystein Handegard <handegar@sim.no>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

##
## A simple Inventor-2-POV converter  
##
## TODO:
##  * Handle transformation of light position
##  * Handle textures
##  * Better lightsource converter (esp. for spotlights)
##  * Native support for POV primitives (Spheres/Cones/Cylinder)
##  * Better camera support
##  * Search graph for lights or cameras BEFORE processing. Add
##    if missing.
##  * Better material handeling
##  * Make it into a library?
##


from __future__ import print_function
from pivy.coin import *
from pivy.sogui import *

##############################################################

lightfound = False

##############################################################
    
def printMaterial(action, idx=0):
    (ambient, diffuse, specular, emissive, shininess, transparency) = action.getMaterial(idx)
    print(" texture { // Material")
    if transparency != 0:
        print("  pigment { color rgbt <%f, %f, %f, %f> }" % (diffuse[0], diffuse[1], diffuse[2], transparency))
    else:
        print("  pigment { color rgb <%f, %f, %f> }" % (diffuse[0], diffuse[1], diffuse[2]))
    ambientfactor = SbVec3f(ambient[0], ambient[1], ambient[2]).length() / 3.0
    diffusefactor = SbVec3f(specular[0], specular[1], specular[2]).length() / 3.0                                
    print("  finish { diffuse 0.8 ambient %f specular %f reflection %f }" % (ambientfactor, diffusefactor, shininess))
    print(" }")

##############################################################

def cameraCallback(userdata, action, camera):
    print("\ncamera {")
    print(" perspective")
    print(" up <0, 1, 0>")
    ## FIXME: This is not really needed (?)  (20050819 handegar)
    ##print " right <%f, 0, 0>" % camera.aspectRatio.getValue()
    print(" direction <0, 0, -1>")
    campos = camera.position.getValue()
    print(" location <%f, %f, %f>" % (campos[0], campos[1], campos[2]))
    print(" angle %f" % (camera.heightAngle.getValue() * (360.0 / (2*M_PI))))                      
    lookat = camera.getViewVolume().getSightPoint(10)
    print(" look_at <%f, %f, %f>" % (lookat[0], lookat[1], lookat[2]))
    print("}\n")



def lightCallback(userdata, action, light):
    global lightfound
    lightfound = True    
    print("\nlight_source {")
    position = SbVec3f()
    if light.isOfType(SoDirectionalLight.getClassTypeId()) :
        position = light.direction.getValue() ## Not exactly correct    
    else :
        position = light.location.getValue()
    print(" <%f, %f, %f>" % (position[0], position[1], position[2]))
    color = light.color.getValue()
    print(" rgb <%f, %f, %f>" % (color[0], color[1], color[2]))
    if light.isOfType(SoDirectionalLight.getClassTypeId()) :
        print(" parallel")
        print(" point_at <0, 0, 0>") ## I'm not sure if this is correct (but it looks OK)
    if light.isOfType(SoSpotLight.getClassTypeId()):
        target = position + light.direction.getValue()
        print(" spotlight")
        print(" radius %f" % (2*(light.cutOffAngle.getValue() * 360) / (2*M_PI)))
        print(" point_at <%f, %f, %f>" % (target[0], target[1], target[2]))        
    print("}\n")



def preShapeCallback(userdata, action, node):
    print("\n// Mesh start")
    if node.getName().getString() != "":
        print("// name: '%s'" % node.getName().getString())    
    print("mesh {")
    return SoCallbackAction.CONTINUE



def postShapeCallback(userdata, action, node):
    printMaterial(action)
    print("} // Mesh end\n")
    return SoCallbackAction.CONTINUE



def triangleCallback(userdata, action, v1, v2, v3):
    matrix = action.getModelMatrix()
    revmatrix = matrix.inverse().transpose()
    p1 = matrix.multVecMatrix(v1.getPoint())
    p2 = matrix.multVecMatrix(v2.getPoint())
    p3 = matrix.multVecMatrix(v3.getPoint())
    n1 = revmatrix.multVecMatrix(v1.getNormal())
    n2 = revmatrix.multVecMatrix(v2.getNormal())
    n3 = revmatrix.multVecMatrix(v3.getNormal())

    ## FIXME: There must be a better way to detect if normals
    ## are invalid than this (20050819 handegar)
    if n1[0] < 2.0: ## Substitute for +NaN as the normal is always normalized
        print(" smooth_triangle {")
        print("  <%f, %f, %f>,<%f, %f, %f>, <%f, %f, %f>,<%f, %f, %f>, <%f, %f, %f>,<%f, %f, %f>" % \
              (p1[0], p1[1], p1[2], n1[0], n1[1], n1[2], p2[0], p2[1], p2[2], n2[0], n2[1], n2[2], p3[0], p3[1], p3[2], n3[0], n3[1], n3[2]))
    else:
        print(" triangle {")
        print("  <%f, %f, %f>, <%f, %f, %f>, <%f, %f, %f>" % (p1[0], p1[1], p1[2], p2[0], p2[1], p2[2], p3[0], p3[1], p3[2]))
    if (action.getMaterialBinding() == SoMaterialBinding.PER_FACE) or \
       (action.getMaterialBinding() == SoMaterialBinding.PER_FACE_INDEXED):
        print("  texture { T%d }" % (v1.getMaterialIndex()))
    print(" }") 
    return SoCallbackAction.CONTINUE



def materialCallback(userdata, action, node):
    print("// Material declarations")
    nr = node.diffuseColor.getNum()    
    for i in range(0, nr):
        d = node.diffuseColor[i]
        ambientfactor = SbVec3f(node.ambientColor[i][0], node.ambientColor[i][1], node.ambientColor[i][2]).length() / 3.0
        specularfactor = SbVec3f(node.specularColor[i][0], node.specularColor[i][1], node.specularColor[i][2]).length() / 3.0
        transparency = node.transparency[i]
        shininess = node.shininess[i]

        if node.ambientColor.getNum() < nr: ambientfactor = 0.2
        if node.specularColor.getNum() < nr: specularfactor = 0
        if node.transparency.getNum() < nr: transparency = 0
        if node.shininess.getNum() < nr: shininess = 0

        print("#declare T%d=" % (i))
        print("texture { pigment { color rgbt <%f, %f, %f, %f> }" % (d[0], d[1], d[2], transparency))
        print("          finish { diffuse 0.8 ambient %f specular %f reflection %f } }" \
              % (ambientfactor, specularfactor, shininess))
        print("// end")



def convert(root):
    print("""
/*
   iv2pov.py  -  An Inventor to Persistence of Vision converter
   Version 0.01 alpha      
   Øystein Handegard, <handegar@sim.no>
*/""")
    callbackAction = SoCallbackAction()
    callbackAction.addPreCallback(SoPerspectiveCamera.getClassTypeId(), cameraCallback, None)
    callbackAction.addPreCallback(SoMaterial.getClassTypeId(), materialCallback, None)    
    callbackAction.addPreCallback(SoLight.getClassTypeId(), lightCallback, None)
    callbackAction.addPreCallback(SoShape.getClassTypeId(), preShapeCallback, None)
    callbackAction.addPostCallback(SoShape.getClassTypeId(), postShapeCallback, None)
    callbackAction.addTriangleCallback(SoShape.getClassTypeId(), triangleCallback, None)
    callbackAction.apply(root)
    
##############################################################

def main():
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    sys.stderr.write("Inventor => POV converter - Version 0.01 alpha\n")
    if len(sys.argv) != 2:
        sys.stderr.write(" Usage: iv2pov.py [iv file] > out.pov\n")
        sys.exit(1)
        
    myInput = SoInput()
    if not myInput.openFile(sys.argv[1]):
        sys.stderr.write(" Could not open specified file.\n")
        sys.exit(1)
    root = SoDB.readAll(myInput)

    sys.stderr.write("* Select a camera angle and press 'q'\n")
    
    # setup viewer
    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Inventor to POV converter")
    myViewer.viewAll()
    myViewer.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

    sys.stderr.write("* Processing...\n")
    cam = myViewer.getCamera()
    root.insertChild(cam, 0)
    convert(root)
    sys.stderr.write("* ...finished\n")


    # Add a default headlight if no light were processed (or else
    # the scene gets completely dark...)
    global lightfound    
    if lightfound != True:
        sys.stderr.write("* Scene contained no lights. Added a headlight to the camera.\n")
        pos = cam.position.getValue()
        print("// Default headlight")
        print("light_source { <%f, %f, %f>, rgb <1, 1, 1> }" % (pos[0], pos[1], pos[2]))
        
    return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# Pivy Quarter unit test suite
#
# For detailed info on its usage and on how to write additional test cases
# read:
#   - http://pyunit.sourceforge.net/pyunit.html
#   - http://diveintopython.org/unit_testing/
#
# Invoke this script with '--help' for usage information.
#

from pivy.quarter import *
import unittest

class Quarter(unittest.TestCase):
    """tests various methods of Quarter"""
    def testSomeQuartermethod(self):
        """check a Quarter method"""
        self.assertTrue(1, 1)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 11, example 1.
#
# Example of reading from a file.
# Read a file given a filename and return a separator
# containing all of the file.  Return NULL if there is 
# an error reading the file.
#

from __future__ import print_function
import sys

from pivy.coin import *
from pivy.sogui import *

#############################################################
# CODE FOR The Inventor Mentor STARTS HERE

def readFile(filename):
    # Open the input file
    mySceneInput = SoInput()
    if not mySceneInput.openFile(filename):
        print("Cannot open file %s" % (filename), file=sys.stderr)
        return None

    # Read the whole file into the database
    myGraph = SoDB.readAll(mySceneInput)
    if myGraph == None:
        print("Problem reading file", file=sys.stderr)
        return None
    
    mySceneInput.closeFile()
    return myGraph

# CODE FOR The Inventor Mentor ENDS HERE
#############################################################

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])

    # Read the file
    scene = readFile("bookshelf.iv")

    # Create a viewer
    myViewer = SoGuiExaminerViewer(myWindow)

    # attach and show viewer
    myViewer.setSceneGraph(scene)
    myViewer.setTitle("File Reader")
    myViewer.show()
    
    # Loop forever
    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



import sys
from PySide2 import QtGui, QtWidgets
from pivy import quarter, coin
from OpenGL.GL import *


class CapPlane():
    def __init__(self, plane, node):
        self.plane = plane  # SbPlane
        self.node = node
        self.normal = plane.getNormal()
        self.dist = plane.getDistanceFromOrigin()

    def stencilBuffer(self):

        eq = GLdouble_4(*self.normal, self.dist)
        glClipPlane(GL_CLIP_PLANE0, eq)

        glEnable(GL_CLIP_PLANE0)

        glEnable(GL_STENCIL_TEST)
        glClear(GL_STENCIL_BUFFER_BIT)

        # Draw Front faces and decrease stencil-buffer
        # where the front_face is drawn
        glStencilFunc(GL_ALWAYS, 0, 0);
        glStencilOp(GL_KEEP, GL_KEEP, GL_DECR)
        glCullFace(GL_FRONT)
        glEnable(GL_CULL_FACE)
        self.draw_box()

        # draw the back_face to the stencil-buffer
        # increase the stencil buffer where the back-face is drawn
        glCullFace(GL_BACK)
        glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE)
        glStencilOp(GL_KEEP, GL_KEEP, GL_INCR)
        self.draw_box()

        # Draw a quad aligned with the stencil plane,
        # but set the stencil test to reject pixels unless the stencil is set.
        glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE)
        glEnable(GL_DEPTH_TEST)
        glDisable(GL_CLIP_PLANE0)
        glDisable(GL_CULL_FACE)
        glStencilFunc(GL_NOTEQUAL, 0, ~0)

        glColor3f(0.1, 0.2, 1)
        glBegin(GL_QUADS)
        glVertex3fv([-1.1,-1.1, 0])
        glVertex3fv([ 1.1,-1.1, 0])
        glVertex3fv([ 1.1, 1.1, 0])
        glVertex3fv([-1.1, 1.1, 0])
        glEnd()

        glDisable(GL_STENCIL_TEST)
        glClear(GL_STENCIL_BUFFER_BIT)

    def draw_box(self):
        glDisable(GL_LIGHTING)
        glPushMatrix()
        glColor3f(0.0, 0.7, 0.0)
        glBegin(GL_QUADS)
        glVertex3fv([-1,-1, 1])
        glVertex3fv([-1, 1, 1])
        glVertex3fv([ 1, 1, 1])
        glVertex3fv([ 1,-1, 1])

        glVertex3fv([-1,-1,-1])
        glVertex3fv([ 1,-1,-1])
        glVertex3fv([ 1, 1,-1])
        glVertex3fv([-1, 1,-1])

        glVertex3fv([ 1,-1,-1])
        glVertex3fv([ 1,-1, 1])
        glVertex3fv([ 1, 1, 1])
        glVertex3fv([ 1, 1,-1])

        glVertex3fv([-1,-1,-1])
        glVertex3fv([-1, 1,-1])
        glVertex3fv([-1, 1, 1])
        glVertex3fv([-1,-1, 1])

        glVertex3fv([-1, 1,-1])
        glVertex3fv([ 1, 1,-1])
        glVertex3fv([ 1, 1, 1])
        glVertex3fv([-1, 1, 1])

        glVertex3fv([-1,-1,-1])
        glVertex3fv([-1,-1, 1])
        glVertex3fv([ 1,-1, 1])
        glVertex3fv([ 1,-1,-1])

        glEnd()
        glPopMatrix()
        glEnable(GL_LIGHTING)


def myCallbackRoutine(cap, action):
    global handled
    if not action.isOfType(coin.SoGLRenderAction.getClassTypeId()):
        return
    cap.stencilBuffer()


def main():
    app = QtWidgets.QApplication(sys.argv)
    viewer = quarter.QuarterWidget()
    # build a scene (sphere, cube)
    plane = coin.SbPlane(coin.SbVec3f(0, 0, 1), coin.SbVec3f(0, 0, 0))
    root = coin.SoSeparator()
    myCallback = coin.SoCallback()
    cap = CapPlane(plane, root)
    myCallback.setCallback(myCallbackRoutine, cap)
    root += myCallback, coin.SoSphere()

    viewer.setSceneGraph(root)
    viewer.setBackgroundColor(coin.SbColor(.5, .5, .5))
    viewer.setWindowTitle("GL stencil buffer")

    viewer.show()

    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from The Inventor Mentor,
# chapter 17, example 2.
#
# Example of combining Inventor and OpenGL rendering.
# Create an Inventor render area and draw a red cube 
# and a blue sphere.  Render the floor with OpenGL 
# through a Callback node.
#

import sys

from OpenGL.GL import *

from pivy.coin import *
from pivy.sogui import *

floorObj = []

# Build a scene with two objects and some light
def buildScene(root):
    # Some light
    root.addChild(SoLightModel())
    root.addChild(SoDirectionalLight())

    # A red cube translated to the left and down
    myTrans = SoTransform()
    myTrans.translation = (-2.0, -2.0, 0.0)
    root.addChild(myTrans)

    myMtl = SoMaterial()
    myMtl.diffuseColor = (1.0, 0.0, 0.0)
    root.addChild(myMtl)
   
    root.addChild(SoCube())

    # A blue sphere translated right
    myTrans = SoTransform()
    myTrans.translation = (4.0, 0.0, 0.0)
    root.addChild(myTrans)

    myMtl = SoMaterial()
    myMtl.diffuseColor = (0.0, 0.0, 1.0)
    root.addChild(myMtl)
   
    root.addChild(SoSphere())

# Build the floor that will be rendered using OpenGL.
def buildFloor():
    global floorObj
    a = 0

    for i in range(9):
        for j in range(9):
            floorObj.append([-5.0 + j*1.25, 0.0, -5.0 + i*1.25])
            a+=1
            
# Draw the lines that make up the floor, using OpenGL
def drawFloor():
    global floorObj

    glBegin(GL_LINES)
    for i in range(4):
        glVertex3fv(floorObj[i*18])
        glVertex3fv(floorObj[(i*18)+8])
        glVertex3fv(floorObj[(i*18)+17])
        glVertex3fv(floorObj[(i*18)+9])
    i+=1
    glVertex3fv(floorObj[i*18])
    glVertex3fv(floorObj[(i*18)+8])
    glEnd()

    glBegin(GL_LINES)
    for i in range(4):
        glVertex3fv(floorObj[i*2])
        glVertex3fv(floorObj[(i*2)+72])
        glVertex3fv(floorObj[(i*2)+73])
        glVertex3fv(floorObj[(i*2)+1])

    i+=1
    glVertex3fv(floorObj[i*2])
    glVertex3fv(floorObj[(i*2)+72])
    glEnd()

# Callback routine to render the floor using OpenGL
def myCallbackRoutine(void, action):
    # only render the floor during GLRender actions:
    if not action.isOfType(SoGLRenderAction.getClassTypeId()): return
   
    glPushMatrix()
    glTranslatef(0.0, -3.0, 0.0)
    glColor3f(0.0, 0.7, 0.0)
    glLineWidth(2)
    glDisable(GL_LIGHTING)  # so we don't have to set normals
    drawFloor()
    glEnable(GL_LIGHTING)   
    glLineWidth(1)
    glPopMatrix()
   
    # With Inventor 2.1, it's necessary to reset SoGLLazyElement after
    # making calls (such as glColor3f()) that affect material state.
    # In this case, the diffuse color and light model are being modified,
    # so the logical-or of DIFFUSE_MASK and LIGHT_MODEL_MASK is passed 
    # to SoGLLazyElement::reset().  
    # Additional information can be found in the publication
    # "Open Inventor 2.1 Porting and Performance Tips"
  
    # state = action.getState()
    # lazyElt = SoLazyElement.getInstance(state)
    # lazyElt.reset(state, (SoLazyElement.DIFFUSE_MASK)|(SoLazyElement.LIGHT_MODEL_MASK))

def main():
    # Initialize Inventor utilities
    myWindow = SoGui.init("Example 17.1")

    buildFloor()

    # Build a simple scene graph, including a camera and
    # a SoCallback node for performing some GL rendering.
    root = SoSeparator()

    myCamera = SoPerspectiveCamera()
    myCamera.position = (0.0, 0.0, 5.0)
    myCamera.heightAngle = M_PI/2.0  # 90 degrees
    myCamera.nearDistance = 2.0
    myCamera.farDistance = 12.0
    root.addChild(myCamera)

    myCallback = SoCallback()
    myCallback.setCallback(myCallbackRoutine)
    root.addChild(myCallback)

    buildScene(root)
   
    # Initialize an Inventor Qt RenderArea and draw the scene.
    myRenderArea = SoGuiRenderArea(myWindow)
    myRenderArea.setSceneGraph(root)
    myRenderArea.setTitle("OpenGL Callback")
    myRenderArea.setBackgroundColor(SbColor(.8, .8, .8))
    myRenderArea.show()
    drawFloor()
    
    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from The Inventor Mentor,
# chapter 17, example 3.
#
# This example draws the same scene as Example 17.2, 
# but using a GLX window.
#

from __future__ import print_function
import sys, time

from OpenGL.GL import *

from pivy.coin import *
from pivy.sogui import *

WINWIDTH  = 400 
WINHEIGHT = 400 

floorObj = []

# Build a Inventor scene with two objects and some light
def buildScene(root):
    # Some light
    root.addChild(SoLightModel())
    root.addChild(SoDirectionalLight())

    # A red cube translated to the left and down
    myTrans = SoTransform()
    myTrans.translation = (-2.0, -2.0, 0.0)
    root.addChild(myTrans)

    myMtl = SoMaterial()
    myMtl.diffuseColor = (1.0, 0.0, 0.0)
    root.addChild(myMtl)
   
    root.addChild(SoCube())

    # A blue sphere translated right
    myTrans = SoTransform()
    myTrans.translation = (4.0, 0.0, 0.0)
    root.addChild(myTrans)

    myMtl = SoMaterial()
    myMtl.diffuseColor = (0.0, 0.0, 1.0)
    root.addChild(myMtl)
   
    root.addChild(SoSphere())


# Build the floor that will be rendered using OpenGL.
def buildFloor():
    global floorObj
    a = 0

    for i in range(9):
        for j in range(9):
            floorObj.append([-5.0 + j*1.25, 0.0, -5.0 + i*1.25])
            a+=1

# Callback used by GLX window
def waitForNotify(Display, e, arg):
    return e.type == MapNotify and e.xmap.window == arg

# Create and initialize GLX window.
def openWindow():
    attributeList = (GLX_RGBA,
                     GLX_RED_SIZE, 1,
                     GLX_GREEN_SIZE, 1,
                     GLX_BLUE_SIZE, 1,
                     GLX_DEPTH_SIZE, 1,
                     GLX_DOUBLEBUFFER,  
                     None)

    # Open the X display 
    display = XOpenDisplay(0)

    # Initialize the GLX visual and context
    vi = glXChooseVisual(display, DefaultScreen(display), attributeList)
    cx = glXCreateContext(display, vi, 0, GL_TRUE)

    # Create the X color map
    cmap = XCreateColormap(display, RootWindow(display, vi.screen), vi.visual, AllocNone)

    # Create and map the X window
    swa.colormap = cmap
    swa.border_pixel = 0
    swa.event_mask = StructureNotifyMask
    window = XCreateWindow(display, RootWindow(display, vi.screen), 100, 100, WINWIDTH,            
                           WINHEIGHT, 0, vi.depth, InputOutput, vi.visual, 
                           (CWBorderPixel | CWColormap | CWEventMask), swa)
    XMapWindow(display, window)
    XIfEvent(display, event, waitForNotify, window)

    # Attach the GLX context to the window
    glXMakeCurrent(display, window, cx)

    return (display, window)


# Draw the lines that make up the floor, using OpenGL
def drawFloor():
    global floorObj

    glBegin(GL_LINES)
    for i in range(4):
        glVertex3fv(floorObj[i*18])
        glVertex3fv(floorObj[(i*18)+8])
        glVertex3fv(floorObj[(i*18)+17])
        glVertex3fv(floorObj[(i*18)+9])

    i+=1
    glVertex3fv(floorObj[i*18])
    glVertex3fv(floorObj[(i*18)+8])
    glEnd()

    glBegin(GL_LINES)
    for i in range(4):
        glVertex3fv(floorObj[i*2])
        glVertex3fv(floorObj[(i*2)+72])
        glVertex3fv(floorObj[(i*2)+73])
        glVertex3fv(floorObj[(i*2)+1])

    i+=1
    glVertex3fv(floorObj[i*2])
    glVertex3fv(floorObj[(i*2)+72])
    glEnd()

def main():
    # Initialize Inventor
    # SoDB.init() invoked automatically upon coin module import

    # Build a simple scene graph
    root = SoSeparator()
    buildScene(root)

    # Build the floor geometry
    buildFloor()

    # Create and initialize window
    display, window = openWindow()
    glEnable(GL_DEPTH_TEST)
    glClearColor(0.8, 0.8, 0.8, 1.0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

    # Set up the camera using OpenGL.
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(90.0, 1.0, 2.0, 12.0)

    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    glTranslatef(0.0, 0.0, -5.0)

    # Render the floor using OpenGL
    glPushMatrix()
    glTranslatef(0.0, -3.0, 0.0)
    glColor3f(0.7, 0.0, 0.0)
    glLineWidth(2.0)
    glDisable(GL_LIGHTING)
    drawFloor()
    glEnable(GL_LIGHTING)
    glPopMatrix()

    # Render the scene
    myViewport(WINWIDTH, WINHEIGHT)
    myRenderAction = SoGLRenderAction(myViewport)
    
    myRenderAction.apply(root)
    glXSwapBuffers(display, window) 
   
    # With inventor 2.1, it's necessary to reset the lazy element
    # any time GL calls are made outside of inventor.  In this example,
    # between the first and second rendering, the inventor state must
    # have both diffuse color and light model reset, since these are
    # modified by the GLX rendering code.  For more information about
    # the lazy element, see the publication,
    # "Open Inventor 2.1 Porting and Performance Tips"

    # To reset the lazy element, first we obtain the state
    # from the action, then obtain the lazy element from the state, 
    # and finally apply a reset to that lazy element.

    state = myRenderAction.getState()
    lazyElt = SoLazyElement_getInstance(state)
    lazyElt.reset(state, (SoLazyElement.DIFFUSE_MASK)|(SoLazyElement.LIGHT_MODEL_MASK))

    time.sleep(5)
   
    # Rerender the floor using OpenGL again:
    glClearColor(0.8, 0.8, 0.8, 1.0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    
    glPushMatrix()
    glTranslatef(0.0, -3.0, 0.0)
    glColor3f(0.0, 0.7, 0.0)
    glLineWidth(2.0)
    glDisable(GL_LIGHTING)
    drawFloor()
    glEnable(GL_LIGHTING)
    glPopMatrix()
   
    # Redraw the rest of the scene: 
    myRenderAction.apply(root)
    glXSwapBuffers(display, window)

    time.sleep(10) 
    return 0

if __name__ == "__main__":
    print("This example is not functional as it is GLX dependent!")
    sys.exit(1)
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from The Inventor Mentor
# chapter 10, example 1.
#
# The scene graph has 4 objects which may be
# selected by picking with the left mouse button
# (use shift key to extend the selection to more
# than one object).
# 
# Hitting the up arrow key will increase the size of
# each selected object; hitting down arrow will decrease
# the size of each selected object.
#

from __future__ import print_function
import sys

from pivy.coin import *
from pivy.sogui import *

# Global data
cubeTransform, sphereTransform, coneTransform, cylTransform = [None] * 4

# Scale each object in the selection list
def myScaleSelection(selection, sf):
    global cubeTransform, sphereTransform, coneTransform, cylTransform

    # Scale each object in the selection list
    for i in range(selection.getNumSelected()):
        selectedPath = selection.getPath(i)
        xform = None

        # Look for the shape node, starting from the tail of the
        # path.  Once we know the type of shape, we know which
        # transform to modify
        for j in range(selectedPath.getLength()):
            if xform != None: break
            n = selectedPath.getNodeFromTail(j)

            if n.isOfType(SoCube.getClassTypeId()):
                xform = cubeTransform
            elif n.isOfType(SoCone.getClassTypeId()):
                xform = coneTransform
            elif n.isOfType(SoSphere.getClassTypeId()):
                xform = sphereTransform
            elif n.isOfType(SoCylinder.getClassTypeId()):
                xform = cylTransform

        # Apply the scale
        xform.scaleFactor = xform.scaleFactor.getValue() * sf

###############################################################
# CODE FOR The Inventor Mentor STARTS HERE  (part 2)

# If the event is down arrow, then scale down every object 
# in the selection list if the event is up arrow, scale up.
# The userData = selection is the selectionRoot from main().
def myKeyPressCB(selection, eventCB):
    event = eventCB.getEvent()
    # check for the Up and Down arrow keys being pressed
    if SoKeyboardEvent_isKeyPressEvent(event, SoKeyboardEvent.UP_ARROW):
        myScaleSelection(selection, 1.1)
        eventCB.setHandled()
    elif SoKeyboardEvent_isKeyPressEvent(event, SoKeyboardEvent.DOWN_ARROW):
        myScaleSelection(selection, 1.0/1.1)
        eventCB.setHandled()

# CODE FOR The Inventor Mentor ENDS HERE
###############################################################

def main():
    global cubeTransform, sphereTransform, coneTransform, cylTransform
    # Print out usage message
    print("Left mouse button        - selects object")
    print("<shift>Left mouse button - selects multiple objects")
    print("Up and Down arrows       - scale selected objects")

    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None:
        sys.exit(1)

    # Create and set up the selection node
    selectionRoot = SoSelection()
    selectionRoot.policy = SoSelection.SHIFT
   
    # Add a camera and some light
    myCamera = SoPerspectiveCamera()
    selectionRoot.addChild(myCamera)
    selectionRoot.addChild(SoDirectionalLight())

###############################################################
# CODE FOR The Inventor Mentor STARTS HERE  (part 1)

    # An event callback node so we can receive key press events
    myEventCB = SoEventCallback()
    myEventCB.addEventCallback(SoKeyboardEvent.getClassTypeId(), 
                               myKeyPressCB, selectionRoot)
    selectionRoot.addChild(myEventCB)

# CODE FOR The Inventor Mentor ENDS HERE
###############################################################

    # Add some geometry to the scene

    # a red cube
    cubeRoot = SoSeparator()
    cubeMaterial = SoMaterial()
    cubeTransform = SoTransform()
    cubeRoot.addChild(cubeTransform)
    cubeRoot.addChild(cubeMaterial)
    cubeRoot.addChild(SoCube())
    cubeTransform.translation = (-2, 2, 0)
    cubeMaterial.diffuseColor = (.8, 0, 0)
    selectionRoot.addChild(cubeRoot)

    # a blue sphere
    sphereRoot = SoSeparator()
    sphereMaterial = SoMaterial()
    sphereTransform = SoTransform()
    sphereRoot.addChild(sphereTransform)
    sphereRoot.addChild(sphereMaterial)
    sphereRoot.addChild(SoSphere())
    sphereTransform.translation = (2, 2, 0)
    sphereMaterial.diffuseColor = (0, 0, .8)
    selectionRoot.addChild(sphereRoot)

    # a green cone
    coneRoot = SoSeparator()
    coneMaterial = SoMaterial()
    coneTransform = SoTransform()
    coneRoot.addChild(coneTransform)
    coneRoot.addChild(coneMaterial)
    coneRoot.addChild(SoCone())
    coneTransform.translation = (2, -2, 0)
    coneMaterial.diffuseColor = (0, .8, 0)
    selectionRoot.addChild(coneRoot)

    # a magenta cylinder
    cylRoot = SoSeparator()
    cylMaterial = SoMaterial()
    cylTransform = SoTransform()
    cylRoot.addChild(cylTransform)
    cylRoot.addChild(cylMaterial)
    cylRoot.addChild(SoCylinder())
    cylTransform.translation = (-2, -2, 0)
    cylMaterial.diffuseColor = (.8, 0, .8)
    selectionRoot.addChild(cylRoot)

    # Create a render area for viewing the scene
    myRenderArea = SoGuiRenderArea(myWindow)
    myRenderArea.setSceneGraph(selectionRoot)
    
    # need to make a reference like this otherwise SoBoxHighlightRenderAction() gets
    # dereferenced after the myRenderArea.setGLRenderAction() call, resulting in its
    # destructor to be called.
    # i.e.: myRenderArea.setGLRenderAction(SoBoxHighlightRenderAction()) would result
    # in a segfault!
    # in my opinion this should _not_ happen, but it does! :(
    #
    # FIXME: investigate why this is so...
    # myRenderArea.setGLRenderAction(SoBoxHighlightRenderAction())
    boxhra = SoBoxHighlightRenderAction()
    myRenderArea.setGLRenderAction(boxhra)
    
    myRenderArea.redrawOnSelectionChange(selectionRoot)
    myRenderArea.setTitle("Adding Event Callbacks")

    # Make the camera see the whole scene
    viewportRegion = myRenderArea.getViewportRegion()
    myCamera.viewAll(selectionRoot, viewportRegion, 2.0)

    # FIXME: soqt maybe has problems here!
    selectionRoot.addSelectionCallback(myWindow.scheduleRedraw)

    # Show our application window, and loop forever...
    myRenderArea.show()
    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 12, example 1.
#
# Sense changes to a viewer's camera's position.
#

import sys

from pivy.coin import *
from pivy.sogui import *

# Callback that reports whenever the viewer's position changes.
def cameraChangedCB(viewerCamera, sensor):
    cameraPosition = viewerCamera.position.getValue()
    print("Camera position: (%g,%g,%g)" % (cameraPosition[0],
                                           cameraPosition[1],
                                           cameraPosition[2]))
    
def main():
    if len(sys.argv) != 2:
        print("Usage: %s filename.iv" % (sys.argv[0]),  file=sys.stderr) 
        sys.exit(1)

    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    inputFile = SoInput()
    if inputFile.openFile(sys.argv[1]) == 0:
        print("Could not open file %s" % (sys.argv[1]), file=sys.stderr)
        sys.exit(1)
   
    root = SoDB.readAll(inputFile)

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Camera Sensor")
    myViewer.show()

    # Get the camera from the viewer, and attach a 
    # field sensor to its position field:
    camera = myViewer.getCamera()
    mySensor = SoFieldSensor(cameraChangedCB, camera)
    mySensor.attach(camera.position)
    
    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import sys
from pivy.gui.soqt import SoQt
from PySide2.QtGui import QApplication
from PySide2.QtCore import QObject
from PySide2.QtCore import SIGNAL
from PySide2.QtCore import SLOT
from mainwindow import MainWindow

if __name__ == "__main__":

    SoQt.init(None)
    app = QApplication(sys.argv)
    window = MainWindow()
    QObject.connect(app, SIGNAL("lastWindowClosed()"), app, SLOT("quit()"))
    window.show()
    sys.exit(app.exec_())
